@@ -27,7 +27,7 @@
 /**
  * Functions for parsing HTML, specially for TYPO3 processing in relation to TCEmain and Rich Text Editor (RTE)
  *
- * $Id: class.t3lib_parsehtml_proc.php 3514 2008-04-02 09:15:43Z benni $
+ * $Id: class.t3lib_parsehtml_proc.php 4794 2009-01-20 12:21:29Z steffenk $
  * Revised for TYPO3 3.6 December/2003 by Kasper Skaarhoj
  * XHTML compatible.
  *
@@ -110,6 +110,7 @@ class t3lib_parsehtml_proc extends t3lib_parsehtml {
 	var $elRef = '';				// Element reference [table]:[field], eg. "tt_content:bodytext"
 	var $relPath='';				// Relative path
 	var $relBackPath='';			// Relative back-path
+	public $tsConfig = array();		// Current Page TSConfig
 	var $procOptions = '';			// Set to the TSconfig options coming from Page TSconfig
 
 		// Internal, dynamic
@@ -232,14 +233,15 @@ class t3lib_parsehtml_proc extends t3lib_parsehtml {
 	function RTE_transform($value,$specConf,$direction='rte',$thisConfig=array())	{
 
 			// Init:
+		$this->tsConfig = $thisConfig;
 		$this->procOptions = $thisConfig['proc.'];
 		$this->preserveTags = strtoupper(implode(',',t3lib_div::trimExplode(',',$this->procOptions['preserveTags'])));
-		
+
 			// dynamic configuration of blockElementList
 		if ($this->procOptions['blockElementList']) {
 			$this->blockElementList = $this->procOptions['blockElementList'];
 		}
-		
+
 			// Get parameters for rte_transformation:
 		$p = $this->rte_p = t3lib_BEfunc::getSpecConfParametersFromArray($specConf['rte_transform']['parameters']);
 
@@ -309,7 +311,7 @@ class t3lib_parsehtml_proc extends t3lib_parsehtml {
 						case 'ts_transform':
 						case 'css_transform':
 							$value = str_replace(chr(13),'',$value);	// Has a very disturbing effect, so just remove all '13' - depend on '10'
-							$this->allowedClasses = t3lib_div::trimExplode(',',strtoupper($this->procOptions['allowedClasses']),1);
+							$this->allowedClasses = t3lib_div::trimExplode(',', $this->procOptions['allowedClasses'], 1);
 							$value = $this->TS_transform_db($value,$cmd=='css_transform');
 						break;
 						case 'ts_strip':
@@ -1146,7 +1148,7 @@ class t3lib_parsehtml_proc extends t3lib_parsehtml {
 					if (isset($keepTags['span']))		{
 						$classes=array_merge(array(''),$this->allowedClasses);
 						$keepTags['span']=array(
- 							'allowedAttribs' => 'class,style,xml:lang',
+ 							'allowedAttribs' => 'id,class,style,title,lang,xml:lang,dir',
 							'fixAttrib' => Array(
 								'class' => Array (
 									'list' => $classes,
@@ -1277,8 +1279,19 @@ class t3lib_parsehtml_proc extends t3lib_parsehtml {
 
 							// CLASS attribute:
 						if (!$this->procOptions['skipClass'] && strcmp(trim($attribs[0]['class']),''))	{	// Set to whatever value
-							if (!count($this->allowedClasses) || in_array(strtoupper($attribs[0]['class']),$this->allowedClasses))	{
-								$newAttribs['class']=$attribs[0]['class'];
+							if (!count($this->allowedClasses) || in_array($attribs[0]['class'], $this->allowedClasses))	{
+ 								$newAttribs['class'] = $attribs[0]['class'];
+							} else {
+								$classes = t3lib_div::trimExplode(' ', $attribs[0]['class'], true);
+								$newClasses = array();
+								foreach ($classes as $class) {
+									if (in_array($class, $this->allowedClasses)) {
+										$newClasses[] = $class;
+									}
+								}
+								if (count($newClasses)) {
+									$newAttribs['class'] = implode(' ', $newClasses);
+								}
 							}
 						}
 
