@@ -2,7 +2,7 @@
 /***************************************************************
 *  Copyright notice
 *
-*  (c) 1999-2005 Kasper Skaarhoj (kasperYYYY@typo3.com)
+*  (c) 1999-2008 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *
 *  This script is part of the Typo3 project. The Typo3 project is
@@ -27,7 +27,7 @@
 /**
  * Parent class for "Services" classes
  *
- * $Id: class.t3lib_svbase.php 1421 2006-04-10 09:27:15Z mundaun $
+ * $Id: class.t3lib_svbase.php 3439 2008-03-16 19:16:51Z flyguide $
  * TODO: temp files are not removed
  *
  * @author	Ren Fritz <r.fritz@colorcube.de>
@@ -448,14 +448,9 @@ class t3lib_svbase {
 	function readFile ($absFile, $length=0)	{
 		$out = FALSE;
 
-		if ($this->checkInputFile ($absFile)) {
-			if ($fd = fopen ($absFile, 'rb')) {
-				$length = intval($length) ? intval($length) : filesize ($absFile);
-				if ($length > 0) {
-					$out = fread ($fd, $length);
-				}
-				fclose ($fd);
-			} else {
+		if ($this->checkInputFile($absFile)) {
+			$out = file_get_contents($absFile);
+			if ($out===FALSE)	{
 				$this->errorPush(T3_ERR_SV_FILE_READ, 'Can not read from file: '.$absFile);
 			}
 		}
@@ -719,4 +714,4 @@ if (defined("TYPO3_MODE") && $TYPO3_CONF_VARS[TYPO3_MODE]["XCLASS"]["t3lib/class
 }
 */
 
-?>
\ No newline at end of file
+?>
