@@ -2,7 +2,7 @@
 /***************************************************************
 *  Copyright notice
 *
-*  (c) 1999-2005 Kasper Skaarhoj (kasperYYYY@typo3.com)
+*  (c) 1999-2008 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
@@ -27,7 +27,7 @@
 /**
  * Contains base class for creating a browsable array/page/folder tree in HTML
  *
- * $Id: class.t3lib_treeview.php 1421 2006-04-10 09:27:15Z mundaun $
+ * $Id: class.t3lib_treeview.php 3439 2008-03-16 19:16:51Z flyguide $
  * Revised for TYPO3 3.6 November/2003 by Kasper Skaarhoj
  *
  * @author	Kasper Skaarhoj <kasperYYYY@typo3.com>
@@ -128,6 +128,8 @@ class t3lib_treeView {
 	/**
 	 * Needs to be initialized with $GLOBALS['BE_USER']
 	 * Done by default in init()
+	 *
+	 * @var t3lib_beUserAuth
 	 */
 	var $BE_USER='';
 
@@ -238,6 +240,7 @@ class t3lib_treeView {
 		// For record trees:
 	var $ids = Array();				// one-dim array of the uid's selected.
 	var $ids_hierarchy = array();	// The hierarchy of element uids
+	var $orig_ids_hierarchy = array();	// The hierarchy of versioned element uids
 	var $buffer_idH = array();		// Temporary, internal array
 
 		// For FOLDER trees:
@@ -322,7 +325,7 @@ class t3lib_treeView {
 
 
 	/**
-	 * Resets the tree, recs, ids, and ids_hierarchy internal variables. Use it if you need it.
+	 * Resets the tree, recs, ids, ids_hierarchy and orig_ids_hierarchy internal variables. Use it if you need it.
 	 *
 	 * @return	void
 	 */
@@ -331,6 +334,7 @@ class t3lib_treeView {
 		$this->recs = array();
 		$this->ids = array();
 		$this->ids_hierarchy = array();
+		$this->orig_ids_hierarchy = array();
 	}
 
 
@@ -663,7 +667,7 @@ class t3lib_treeView {
 
 
 	/**
-	 * Returns the title for the input record. If blank, a "no title" labele (localized) will be returned.
+	 * Returns the title for the input record. If blank, a "no title" label (localized) will be returned.
 	 * Do NOT htmlspecialchar the string from this function - has already been done.
 	 *
 	 * @param	array		The input row array (where the key "title" is used for the title)
@@ -671,7 +675,16 @@ class t3lib_treeView {
 	 * @return	string		The title.
 	 */
 	function getTitleStr($row,$titleLen=30)	{
-		$title = (!strcmp(trim($row['title']),'')) ? '<em>['.$GLOBALS['LANG']->sL('LLL:EXT:lang/locallang_core.php:labels.no_title',1).']</em>' : htmlspecialchars(t3lib_div::fixed_lgd_cs($row['title'],$titleLen));
+		if ($this->ext_showNavTitle && strlen(trim($row['nav_title'])) > 0)	{
+			$title = '<span title="'.$GLOBALS['LANG']->sL('LLL:EXT:lang/locallang_tca.xml:title',1).' '.htmlspecialchars(trim($row['title'])).'">'.htmlspecialchars(t3lib_div::fixed_lgd_cs($row['nav_title'],$titleLen)).'</span>';
+		} else {
+			$title = htmlspecialchars(t3lib_div::fixed_lgd_cs($row['title'],$titleLen));
+			if (strlen(trim($row['nav_title'])) > 0)	{
+				$title = '<span title="'.$GLOBALS['LANG']->sL('LLL:EXT:cms/locallang_tca.xml:pages.nav_title',1).' '.htmlspecialchars(trim($row['nav_title'])).'">'.$title.'</span>';
+			}
+			$title = (strlen(trim($row['title'])) == 0) ? '<em>['.$GLOBALS['LANG']->sL('LLL:EXT:lang/locallang_core.php:labels.no_title',1).']</em>' : $title;
+		}
+
 		return $title;
 	}
 
@@ -741,7 +754,6 @@ class t3lib_treeView {
 			// Buffer for id hierarchy is reset:
 		$this->buffer_idH=array();
 
-
 			// Init vars
 		$depth=intval($depth);
 		$HTML='';
@@ -756,7 +768,13 @@ class t3lib_treeView {
 			$a++;
 			$crazyRecursionLimiter--;
 
-			$newID =$row['uid'];
+			$newID = $row['uid'];
+
+			if ($newID==0)	{
+				t3lib_BEfunc::typo3PrintError ('Endless recursion detected', 'TYPO3 has detected an error in the database. Please fix it manually (e.g. using phpMyAdmin) and change the UID of '.$this->table.':0 to a new value.<br /><br />See <a href="http://bugs.typo3.org/view.php?id=3495" target="_blank">bugs.typo3.org/view.php?id=3495</a> to get more information about a possible cause.',0);
+				exit;
+			}
+
 			$this->tree[]=array();		// Reserve space.
 			end($this->tree);
 			$treeKey = key($this->tree);	// Get the key for this space
@@ -768,8 +786,9 @@ class t3lib_treeView {
 			}
 
 				// Accumulate the id of the element in the internal arrays
-			$this->ids[]=$idH[$row['uid']]['uid'] = $row['uid'];
+			$this->ids[] = $idH[$row['uid']]['uid'] = $row['uid'];
 			$this->ids_hierarchy[$depth][] = $row['uid'];
+			$this->orig_ids_hierarchy[$depth][] = $row['_ORIG_uid'] ? $row['_ORIG_uid'] : $row['uid'];
 
 				// Make a recursive call to the next level
 			$HTML_depthData = $depthData.'<img'.t3lib_iconWorks::skinImg($this->backPath,'gfx/ol/'.$LN.'.gif','width="18" height="16"').' alt="" />';
@@ -891,7 +910,7 @@ class t3lib_treeView {
 	 * For arrays: This will return key to the ->dataLookup array
 	 *
 	 * @param	integer		parent item id
-	 * @param	string		Class for sub-elements. 
+	 * @param	string		Class for sub-elements.
 	 * @return	mixed		data handle (Tables: An sql-resource, arrays: A parentId integer. -1 is returned if there were NO subLevel.)
 	 * @access private
 	 */
@@ -944,7 +963,7 @@ class t3lib_treeView {
 	 * @access private
 	 * @see getDataInit()
 	 */
-	function getDataNext(&$res,$subCSSclass=''){
+	function getDataNext(&$res,$subCSSclass='')	{
 		if (is_array($this->data)) {
 			if ($res<0) {
 				$row=FALSE;
@@ -958,8 +977,10 @@ class t3lib_treeView {
 			}
 			return $row;
 		} else {
-			$row = @$GLOBALS['TYPO3_DB']->sql_fetch_assoc($res);
-			t3lib_BEfunc::workspaceOL($this->table, $row, $this->BE_USER->workspace);
+			while($row = @$GLOBALS['TYPO3_DB']->sql_fetch_assoc($res))	{
+				t3lib_BEfunc::workspaceOL($this->table, $row, $this->BE_USER->workspace, TRUE);
+				if (is_array($row))	break;
+			}
 
 				// Passing on default <td> class for subelements:
 			if (is_array($row) && $subCSSclass!=='')	{
