@@ -27,7 +27,7 @@
 /**
  * Page navigation tree for the Web module
  *
- * $Id: alt_db_navframe.php 3775 2008-06-10 12:05:31Z patrick $
+ * $Id: alt_db_navframe.php 4571 2008-12-18 18:38:49Z steffenk $
  * Revised for TYPO3 3.6 2/2003 by Kasper Skaarhoj
  * XHTML compliant
  *
@@ -167,11 +167,19 @@ class SC_alt_db_navframe {
 
 		// Function, loading the list frame from navigation tree:
 		function jumpTo(id, linkObj, highlightID, bank)	{ //
-			var theUrl = top.TS.PATH_typo3 + top.currentSubScript + "?id=" + id;
+			var theUrl = top.TS.PATH_typo3 + top.currentSubScript ;
+			if (theUrl.indexOf("?") != -1) {
+				theUrl += "&id=" + id
+			} else {
+				theUrl += "?id=" + id		    	
+			}	
 			top.fsMod.currentBank = bank;
 
-			if (top.condensedMode) top.content.location.href = theUrl;
-			else                   parent.list_frame.location.href=theUrl;
+			if (top.condensedMode) {
+				top.content.location.href = theUrl;
+			} else {
+				parent.list_frame.location.href=theUrl;
+			}
 
 			'.($this->doHighlight ? 'Tree.highlightActiveItem("web", highlightID + "_" + bank);' : '').'
 			'.(!$GLOBALS['CLIENT']['FORMSTYLE'] ? '' : 'if (linkObj) linkObj.blur(); ').'
