@@ -30,7 +30,7 @@
  * Provides links to registered shortcuts
  * If the 'cms' extension is loaded you will also have a field for entering page id/alias which will be found/edited
  *
- * $Id: alt_shortcut.php 3510 2008-04-01 20:42:57Z flyguide $
+ * $Id: alt_shortcut.php 4623 2008-12-29 13:53:48Z steffenk $
  * Revised for TYPO3 3.6 2/2003 by Kasper Skaarhoj
  * XHTML compliant output
  *
@@ -168,7 +168,7 @@ class SC_alt_shortcut {
 		$url = urldecode($this->URL);
 
 			// Lookup the title of this page and use it as default description
-		$page_id = $this->getLinkedPageId($url);
+		$page_id = $this->getLinkedPageId($url);         
 		if (t3lib_div::testInt($page_id))	{
 			if (preg_match('/\&edit\[(.*)\]\[(.*)\]=edit/',$url,$matches))	{
 					// Edit record
@@ -267,10 +267,10 @@ class SC_alt_shortcut {
 				window.location.href="alt_shortcut.php?editShortcut="+uid;
 			}
 			function submitEditPage(id)	{	//
-				window.location.href="alt_shortcut.php?editPage="+top.rawurlencode(id);
+				window.location.href="alt_shortcut.php?editPage="+top.rawurlencodeAndRemoveSiteUrl(id);
 			}
 			function changeWorkspace(workspaceId)	{	//
-				window.location.href="alt_shortcut.php?changeWorkspace="+top.rawurlencode(workspaceId);
+				window.location.href="alt_shortcut.php?changeWorkspace="+top.rawurlencodeAndRemoveSiteUrl(workspaceId);
 			}
 			function changeWorkspacePreview(newstate)	{	//
 				window.location.href="alt_shortcut.php?changeWorkspacePreview="+newstate;
