@@ -2,7 +2,7 @@
 /***************************************************************
 *  Copyright notice
 *
-*  (c) 1999-2005 Kasper Skaarhoj (kasperYYYY@typo3.com)
+*  (c) 1999-2008 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
@@ -30,7 +30,7 @@
  * This script lets users choose a new database element to create.
  * Includes a wizard mode for visually pointing out the position of new pages
  *
- * $Id: db_new.php 1421 2006-04-10 09:27:15Z mundaun $
+ * $Id: db_new.php 3775 2008-06-10 12:05:31Z patrick $
  * Revised for TYPO3 3.6 November/2003 by Kasper Skaarhoj
  * XHTML compliant
  *
@@ -133,8 +133,10 @@ class SC_db_new {
 	var $newPagesAfter;
 	var $web_list_modTSconfig;
 	var $allowedNewTables;
+	var $deniedNewTables;
 	var $web_list_modTSconfig_pid;
 	var $allowedNewTables_pid;
+	var $deniedNewTables_pid;
 	var $code;
 	var $R_URI;
 
@@ -145,7 +147,13 @@ class SC_db_new {
 
 		// Internal
 	var $perms_clause;	// see init()
-	var $doc;			// see init()
+
+	/**
+	 * Document template object
+	 *
+	 * @var mediumDoc
+	 */
+	var $doc;
 	var $content;		// Accumulated HTML output
 
 
@@ -172,14 +180,17 @@ class SC_db_new {
 		$this->pagesOnly = t3lib_div::_GP('pagesOnly');
 
 			// Create instance of template class for output
-		$this->doc = t3lib_div::makeInstance('mediumDoc');
+		$this->doc = t3lib_div::makeInstance('template');
 		$this->doc->backPath = $BACK_PATH;
+		$this->doc->setModuleTemplate('templates/db_new.html');
 		$this->doc->docType= 'xhtml_trans';
 		$this->doc->JScode='';
 
+			// Setting up the context sensitive menu:
+		$this->doc->getContextMenuCode();
+
 			// Creating content
 		$this->content='';
-		$this->content.=$this->doc->startPage($LANG->sL('LLL:EXT:lang/locallang_core.php:db_new.php.pagetitle'));
 		$this->content.=$this->doc->header($LANG->sL('LLL:EXT:lang/locallang_core.php:db_new.php.pagetitle'));
 
 			// Id a positive id is supplied, ask for the page record with permission information contained:
@@ -229,24 +240,34 @@ class SC_db_new {
 				// Acquiring TSconfig for this module/current page:
 			$this->web_list_modTSconfig = t3lib_BEfunc::getModTSconfig($this->pageinfo['uid'],'mod.web_list');
 			$this->allowedNewTables = t3lib_div::trimExplode(',',$this->web_list_modTSconfig['properties']['allowedNewTables'],1);
+			$this->deniedNewTables = t3lib_div::trimExplode(',',$this->web_list_modTSconfig['properties']['deniedNewTables'],1);
 
 				// Acquiring TSconfig for this module/parent page:
 			$this->web_list_modTSconfig_pid = t3lib_BEfunc::getModTSconfig($this->pageinfo['pid'],'mod.web_list');
 			$this->allowedNewTables_pid = t3lib_div::trimExplode(',',$this->web_list_modTSconfig_pid['properties']['allowedNewTables'],1);
+			$this->deniedNewTables_pid = t3lib_div::trimExplode(',',$this->web_list_modTSconfig_pid['properties']['deniedNewTables'],1);
 
 				// More init:
 			if (!$this->showNewRecLink('pages'))	{
 				$this->newPagesInto=0;
 			}
-			if (!$this->showNewRecLink('pages',$this->allowedNewTables_pid))	{
+			if (!$this->showNewRecLink('pages', $this->allowedNewTables_pid, $this->deniedNewTables_pid))	{
 				$this->newPagesAfter=0;
 			}
 
 
 				// Set header-HTML and return_url
-			$this->code=$this->doc->getHeader('pages',$this->pageinfo,$this->pageinfo['_thePath']).'<br />
-			';
-			$this->R_URI=$this->returnUrl;
+			if (is_array($this->pageinfo) && $this->pageinfo['uid'])	{
+				$iconImgTag = t3lib_iconWorks::getIconImage('pages', $this->pageinfo, $this->backPath, 'title="' . htmlspecialchars($this->pageinfo['_thePath']) . '"');
+				$title = strip_tags($this->pageinfo[$GLOBALS['TCA']['pages']['ctrl']['label']]);
+			} else {
+				$iconImgTag = '<img' . t3lib_iconWorks::skinImg($this->backPath, 'gfx/i/_icon_website.gif') . ' title="' . htmlspecialchars($this->pageinfo['_thePath']) . '" alt="" />';
+				$title = $GLOBALS['TYPO3_CONF_VARS']['SYS']['sitename'];
+			}
+
+			$this->code = '<span class="typo3-moduleHeader">' . $this->doc->wrapClickMenuOnIcon($iconImgTag, 'pages', $this->pageinfo['uid']) . htmlspecialchars(t3lib_div::fixed_lgd_cs($title, 45)) . '</span><br />';
+
+			$this->R_URI = $this->returnUrl;
 
 				// GENERATE the HTML-output depending on mode (pagesOnly is the page wizard)
 			if (!$this->pagesOnly)	{	// Regular new element:
@@ -255,19 +276,81 @@ class SC_db_new {
 				$this->pagesOnly();
 			}
 
-				// Create go-back link.
-			if ($this->R_URI)	{
-				$this->code.='<br />
-		<a href="'.htmlspecialchars($this->R_URI).'" class="typo3-goBack">'.
-		'<img'.t3lib_iconWorks::skinImg($this->doc->backPath,'gfx/goback.gif','width="14" height="14"').' alt="" />'.
-		$LANG->getLL('goBack',1).
-		'</a>';
-			}
 				// Add all the content to an output section
 			$this->content.=$this->doc->section('',$this->code);
+
+							// Setting up the buttons and markers for docheader
+			$docHeaderButtons = $this->getButtons();
+			$markers['CSH'] = $docHeaderButtons['csh'];
+
+			$markers['CONTENT'] = $this->content;
+
+				// Build the <body> for the module
+			$this->content = $this->doc->startPage($LANG->sL('LLL:EXT:lang/locallang_core.php:db_new.php.pagetitle'));
+			$this->content.= $this->doc->moduleBody($this->pageinfo, $docHeaderButtons, $markers);
+			$this->content.= $this->doc->endPage();
+			$this->content = $this->doc->insertStylesAndJS($this->content);
 		}
 	}
 
+	/**
+	 * Create the panel of buttons for submitting the form or otherwise perform operations.
+	 *
+	 * @return	array	all available buttons as an assoc. array
+	 */
+	protected function getButtons()	{
+		global $LANG, $BACK_PATH;
+
+		$buttons = array(
+			'csh' => '',
+			'back' => '',
+			'view' => '',
+			'new_page' => '',
+			'record_list' => ''
+		);
+
+
+		if (!$this->pagesOnly)	{	// Regular new element:
+				// New page
+			if ($this->showNewRecLink('pages'))	{
+				$buttons['new_page'] = '<a href="' . htmlspecialchars(t3lib_div::linkThisScript(array('pagesOnly' => '1'))) . '">' .
+					'<img' . t3lib_iconWorks::skinImg($this->doc->backPath, 'gfx/new_page.gif') . ' alt="" />' .
+					'</a>';
+			}
+				// CSH
+			$buttons['csh'] = t3lib_BEfunc::cshItem('xMOD_csh_corebe', 'new_regular', $GLOBALS['BACK_PATH'], '', TRUE);
+		} elseif($this->showNewRecLink('pages')) {	// Pages only wizard
+				// CSH
+			$buttons['csh'] = t3lib_BEfunc::cshItem('xMOD_csh_corebe', 'new_pages', $GLOBALS['BACK_PATH'], '', TRUE);
+		}
+
+			// Back
+		if ($this->R_URI) {
+			$buttons['back'] = '<a href="' . htmlspecialchars($this->R_URI) . '" class="typo3-goBack">' .
+				'<img' . t3lib_iconWorks::skinImg($this->doc->backPath, 'gfx/goback.gif') . ' alt="" />' .
+				'</a>';
+		}
+
+		if (is_array($this->pageinfo) && $this->pageinfo['uid']) {
+				// View
+			$buttons['view'] = '<a href="#" onclick="' . htmlspecialchars(t3lib_BEfunc::viewOnClick($this->pageinfo['uid'], $this->backPath, t3lib_BEfunc::BEgetRootLine($this->pageinfo['uid']))) . '">' .
+				'<img' . t3lib_iconWorks::skinImg($this->backPath, 'gfx/zoom.gif') . ' title="' . $GLOBALS['LANG']->sL('LLL:EXT:lang/locallang_core.php:labels.showPage', 1) . '" alt="" />' .
+				'</a>';
+
+				// Record list
+			if ($GLOBALS['BE_USER']->check('modules', 'web_list')) {
+				$href = $this->backPath . 'db_list.php?id=' . $this->pageinfo['uid'] . '&returnUrl=' . rawurlencode(t3lib_div::getIndpEnv('REQUEST_URI'));
+				$buttons['record_list'] = '<a href="' . htmlspecialchars($href) . '">' .
+					'<img' . t3lib_iconWorks::skinImg($this->backPath, 'gfx/list.gif') . ' title="' . $GLOBALS['LANG']->sL('LLL:EXT:lang/locallang_core.php:labels.showList', 1) . '" alt="" />' .
+					'</a>';
+			}
+		}
+
+
+
+		return $buttons;
+	}
+
 	/**
 	 * Creates the position map for pages wizard
 	 *
@@ -281,9 +364,6 @@ class SC_db_new {
 			<h3>'.htmlspecialchars($LANG->getLL('selectPosition')).':</h3>
 		';
 		$this->code.= $posMap->positionTree($this->id,$this->pageinfo,$this->perms_clause,$this->R_URI);
-
-				// Add CSH:
-		$this->code.= t3lib_BEfunc::cshItem('xMOD_csh_corebe', 'new_pages', $GLOBALS['BACK_PATH'],'<br/>');
 	}
 
 	/**
@@ -364,7 +444,7 @@ class SC_db_new {
 							$overrideExt = $this->web_list_modTSconfig['properties']['newContentWiz.']['overrideWithExtension'];
 							$pathToWizard = (t3lib_extMgm::isLoaded($overrideExt)) ? (t3lib_extMgm::extRelPath($overrideExt).'mod1/db_new_content_el.php') : 'sysext/cms/layout/db_new_content_el.php';
 
-							$href = $pathToWizard.'?id='.$this->id.'&returnUrl='.rawurlencode($this->R_URI);
+							$href = $pathToWizard.'?id='.$this->id.'&returnUrl='.rawurlencode(t3lib_div::getIndpEnv('REQUEST_URI'));
 							$rowContent.= '<br /><img'.t3lib_iconWorks::skinImg($this->doc->backPath,'gfx/ol/line.gif','width="18" height="16"').' alt="" />'.
 										'<img'.t3lib_iconWorks::skinImg($this->doc->backPath,'gfx/ol/joinbottom.gif','width="18" height="16"').' alt="" />'.
 										'<a href="'.htmlspecialchars($href).'"><img'.t3lib_iconWorks::skinImg($this->doc->backPath,'gfx/new_record.gif','width="16" height="12"').' alt="" /> '.
@@ -429,25 +509,6 @@ class SC_db_new {
 			'.implode('',$tRows).'
 			</table>
 		';
-
-			// Create a link to the new-pages wizard.
-		if ($this->showNewRecLink('pages'))	{
-			$this->code.='
-
-				<!--
-					Link; create new page:
-				-->
-				<div id="typo3-newPageLink">
-					<a href="'.htmlspecialchars(t3lib_div::linkThisScript(array('pagesOnly'=>'1'))).'">'.
-					'<img'.t3lib_iconWorks::skinImg($this->doc->backPath,'gfx/new_page.gif','width="13" height="12"').' alt="" />'.
-					htmlspecialchars($LANG->getLL('createNewPage')).
-					'</a>
-				</div>
-				';
-		}
-
-			// Add CSH:
-		$this->code.= t3lib_BEfunc::cshItem('xMOD_csh_corebe', 'new_regular', $GLOBALS['BACK_PATH'],'<br/>');
 	}
 
 	/**
@@ -456,8 +517,6 @@ class SC_db_new {
 	 * @return	void
 	 */
 	function printContent()	{
-		$this->content.= $this->doc->endPage();
-		$this->content = $this->doc->insertStylesAndJS($this->content);
 		echo $this->content;
 	}
 
@@ -514,15 +573,31 @@ class SC_db_new {
 	}
 
 	/**
-	 * Returns true if the $table tablename is found in $allowedNewTables (or if $allowedNewTables is empty)
+	 * Returns true if:
+	 * - $allowedNewTables and $deniedNewTables are empty
+	 * - the table is not found in $deniedNewTables and $allowedNewTables is not set or the $table tablename is found in $allowedNewTables
+	 *
+	 * If $table tablename is found in $allowedNewTables and $deniedNewTables, $deniedNewTables
+	 * has priority over $allowedNewTables.
 	 *
 	 * @param	string		Table name to test if in allowedTables
 	 * @param	array		Array of new tables that are allowed.
-	 * @return	boolean		Returns true if the $table tablename is found in $allowedNewTables (or if $allowedNewTables is empty)
+	 * @param	array		Array of new tables that are not allowed.
+	 * @return	boolean		Returns true if a link for creating new records should be displayed for $table
 	 */
-	function showNewRecLink($table,$allowedNewTables='')	{
-		$allowedNewTables = is_array($allowedNewTables) ? $allowedNewTables : $this->allowedNewTables;
-		return !count($allowedNewTables) || in_array($table,$allowedNewTables);
+	function showNewRecLink($table, array $allowedNewTables=array(), array $deniedNewTables=array()) {
+		$allowedNewTables = ($allowedNewTables ? $allowedNewTables : $this->allowedNewTables);
+		$deniedNewTables = ($deniedNewTables ? $deniedNewTables : $this->deniedNewTables);
+			// No deny/allow tables are set:
+		if (!count($allowedNewTables) && !count($deniedNewTables)) {
+			return true;
+			// If table is not denied (which takes precedence over allowed tables):
+		} elseif (!in_array($table, $deniedNewTables) && (!count($allowedNewTables) || in_array($table, $allowedNewTables))) {
+			return true;
+			// If table is denied or allowed tables are set, but table is not part of:
+		} else {
+			return false;
+		}
 	}
 }
 
@@ -540,4 +615,4 @@ $SOBE = t3lib_div::makeInstance('SC_db_new');
 $SOBE->init();
 $SOBE->main();
 $SOBE->printContent();
-?>
\ No newline at end of file
+?>
