@@ -2,7 +2,7 @@
 /***************************************************************
 *  Copyright notice
 *
-*  (c) 1999-2005 Kasper Skaarhoj (kasperYYYY@typo3.com)
+*  (c) 1999-2008 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
@@ -27,7 +27,7 @@
 /**
  * Shows information about a database or file item
  *
- * $Id: show_rechis.php 1421 2006-04-10 09:27:15Z mundaun $
+ * $Id: show_rechis.php 3775 2008-06-10 12:05:31Z patrick $
  * Revised for TYPO3 3.6 November/2003 by Kasper Skaarhoj
  * XHTML Compliant
  *
@@ -77,6 +77,12 @@ class SC_show_rechis {
 
 		// Internal:
 	var $content;
+
+	/**
+	 * Document template object
+	 *
+	 * @var mediumDoc
+	 */
 	var $doc;
 
 	/**
@@ -88,11 +94,12 @@ class SC_show_rechis {
 		global $LANG;
 
 			// Create internal template object:
-		$this->doc = t3lib_div::makeInstance('mediumDoc');
+		$this->doc = t3lib_div::makeInstance('template');
+		$this->doc->backPath = $GLOBALS['BACK_PATH'];
 		$this->doc->docType = 'xhtml_trans';
+		$this->doc->setModuleTemplate('templates/show_rechis.html');
 
 			// Start the page header:
-		$this->content.=$this->doc->startPage($LANG->getLL('title'));
 		$this->content.=$this->doc->header($LANG->getLL('title'));
 		$this->content.=$this->doc->spacer(5);
 	}
@@ -108,19 +115,17 @@ class SC_show_rechis {
 			// Start history object
 		$historyObj = t3lib_div::makeInstance('recordHistory');
 
-			// Return link:
-		if ($historyObj->returnUrl)	{
-			$this->content .= '<a href="'.htmlspecialchars($historyObj->returnUrl).'" class="typo3-goBack"><img'.t3lib_iconWorks::skinImg('','gfx/goback.gif','width="14" height="14"').' alt="" />'.$LANG->getLL('returnLink',1).'</a>';
-		}
-
 			// Get content:
 		$this->content .= $historyObj->main();
 
-			// Return link:
-		if ($historyObj->returnUrl)	{
-			$link = '<a href="'.htmlspecialchars($historyObj->returnUrl).'" class="typo3-goBack"><img'.t3lib_iconWorks::skinImg('','gfx/goback.gif','width="14" height="14"').' alt="" />'.$LANG->getLL('returnLink',1).'</a>';
-			$this->content .= $this->doc->section($LANG->getLL('return'),$link,0,1);
-		}
+			// Setting up the buttons and markers for docheader
+		$docHeaderButtons = $this->getButtons();
+		$markers['CONTENT'] = $this->content;
+		$markers['CSH'] = $docHeaderButtons['csh'];
+
+			// Build the <body> for the module
+		$this->content = $this->doc->startPage($LANG->getLL('title'));
+		$this->content.= $this->doc->moduleBody($this->pageinfo, $docHeaderButtons, $markers);
 	}
 
 	/**
@@ -134,6 +139,30 @@ class SC_show_rechis {
 		$this->content = $this->doc->insertStylesAndJS($this->content);
 		echo $this->content;
 	}
+
+	/**
+	 * Create the panel of buttons for submitting the form or otherwise perform operations.
+	 *
+	 * @return	array	all available buttons as an assoc. array
+	 */
+	protected function getButtons() {
+		$buttons = array(
+			'csh' => '',
+			'back' => ''
+		);
+
+			// CSH
+		$buttons['csh'] = t3lib_BEfunc::cshItem('xMOD_csh_corebe', 'history_log', $GLOBALS['BACK_PATH'], '', TRUE);
+
+			// Start history object
+		$historyObj = t3lib_div::makeInstance('recordHistory');
+
+		if ($historyObj->returnUrl)	{
+			$buttons['back']= '<a href="' . htmlspecialchars($historyObj->returnUrl) . '" class="typo3-goBack"><img' . t3lib_iconWorks::skinImg('', 'gfx/goback.gif') . ' alt="" title="' . $GLOBALS['LANG']->getLL('returnLink', 1) . '" /></a>';
+		}
+
+		return $buttons;
+	}
 }
 
 // Include extension?
@@ -157,4 +186,4 @@ $SOBE = t3lib_div::makeInstance('SC_show_rechis');
 $SOBE->init();
 $SOBE->main();
 $SOBE->printContent();
-?>
\ No newline at end of file
+?>
