@@ -1317,7 +1317,7 @@ class SC_mod_tools_be_user_index {
 		} else {
 			$menu=array();
 			foreach ($options as $kk => $vv) {
-				$menu[]='<input type="checkbox" value="1" name="compareFlags['.$kk.']" id="checkCompare_'.$kk.'"'.($compareFlags[$kk]?' checked="checked"':'').'> <label for="checkCompare_'.$kk.'">'.htmlspecialchars($vv).'</label>';
+				$menu[]='<input type="checkbox" class="checkbox" value="1" name="compareFlags['.$kk.']" id="checkCompare_'.$kk.'"'.($compareFlags[$kk]?' checked="checked"':'').'> <label for="checkCompare_'.$kk.'">'.htmlspecialchars($vv).'</label>';
 			}
 			$outCode = 'Group by:<br />'.implode('<br />',$menu);
 			$content = $this->doc->section('Group and Compare Users',$outCode,0,1);
@@ -1555,14 +1555,22 @@ class SC_mod_tools_be_user_index {
 			// Process and visualized each active session as a table row:
 		if (is_array($sessions)) {
 			foreach ($sessions as $session) {
-				$hostName = ($session['ses_iplock'] != '[DISABLED]' ? gethostbyaddr($session['ses_iplock']) : '[DISABLED]');
+				$ip = $session['ses_iplock'];
+				$hostName = '';
+				if ($session['ses_iplock'] == '[DISABLED]' || $GLOBALS['TYPO3_CONF_VARS']['BE']['lockIP'] == 0) {
+					$ip = '-';
+				} elseif ($GLOBALS['TYPO3_CONF_VARS']['BE']['lockIP'] == 4) {
+					$hostName = ' title="' . @gethostbyaddr($session['ses_iplock']) . '"';
+				} else {
+					$ip .= str_repeat('.*', 4-$GLOBALS['TYPO3_CONF_VARS']['BE']['lockIP']);
+				}
 				$outTable .= '
 					<tr class="bgColor4" height="17" valign="top">' .
 						'<td nowrap="nowrap">' .
 							date($GLOBALS['TYPO3_CONF_VARS']['SYS']['ddmmyy'].' '.$GLOBALS['TYPO3_CONF_VARS']['SYS']['hhmm'], $session['ses_tstamp']) .
 						'</td>' .
 						'<td nowrap="nowrap">' .
-							'<span title="'.$hostName.'">'.$session['ses_iplock'].'</span>' .
+							'<span'.$hostName.'>'.$ip.'</span>' .
 						'</td>' .
 						'<td width="130">' .
 							t3lib_iconWorks::getIconImage('be_users',$session,$GLOBALS['BACK_PATH'],'align="top" title="'.$session['uid'].'"').htmlspecialchars($session['username']).'&nbsp;' .
@@ -1612,4 +1620,4 @@ $SOBE->init();
 $SOBE->main();
 $SOBE->printContent();
 
-?>
\ No newline at end of file
+?>
