@@ -27,7 +27,7 @@
 /**
  * Include file extending db_list.inc for use with the web_layout module
  *
- * $Id: class.tx_cms_layout.php 3562 2008-04-14 15:20:41Z flyguide $
+ * $Id: class.tx_cms_layout.php 4625 2008-12-29 14:01:18Z steffenk $
  * Revised for TYPO3 3.6 November/2003 by Kasper Skaarhoj
  * XHTML compliant
  *
@@ -753,7 +753,6 @@ class tx_cms_layout extends recordList {
 
 		$this->addElement_tdParams=array(
 			'username'=>' nowrap="nowrap"',
-			'password'=>' nowrap="nowrap"',
 			'usergroup'=>' nowrap="nowrap"',
 			'name'=>' nowrap="nowrap"',
 			'address'=>' nowrap="nowrap"',
@@ -762,7 +761,7 @@ class tx_cms_layout extends recordList {
 			'email'=>' nowrap="nowrap"',
 			'telephone'=>' nowrap="nowrap"'
 		);
-		$fList = 'username,password,usergroup,name,email,telephone,address,zip,city';
+		$fList = 'username,usergroup,name,email,telephone,address,zip,city';
 		$out = $this->makeOrdinaryList('fe_users',$id, $fList, 1);
 		$this->addElement_tdParams=array();
 		return $out;
@@ -1474,7 +1473,7 @@ class tx_cms_layout extends recordList {
 			// Create header row:
 		$out = '
 				<tr>
-					<td class="bgColor2" nowrap="nowrap"><img src="clear.gif" width="1" height="2" alt="" /><br /><div align="center"><b>'.htmlspecialchars(strtoupper($colName)).'</b></div><img src="clear.gif" width="1" height="2" alt="" /></td>
+					<td class="bgColor2" nowrap="nowrap"><img src="clear.gif" width="1" height="2" alt="" /><br /><div align="center"><b>' . htmlspecialchars($GLOBALS['LANG']->csConvObj->conv_case($GLOBALS['LANG']->charSet, $colName, 'toUpper')) . '</b></div><img src="clear.gif" width="1" height="2" alt="" /></td>
 				</tr>';
 
 			// Create command links:
@@ -1668,13 +1667,17 @@ class tx_cms_layout extends recordList {
 		$outHeader='';
 
 			// Make header:
-		if ($row['header'] && $row['header_layout']!=100)	{
+		if ($row['header']) {
 			$infoArr = Array();
 			$this->getProcessedValue('tt_content','header_position,header_layout,header_link',$row,$infoArr);
 
+			// If header layout is set to 'hidden', display an accordant note:
+			if ($row['header_layout'] == 100) {
+				$hiddenHeaderNote = ' <em>[' . $GLOBALS['LANG']->sL('LLL:EXT:lang/locallang_core.xml:labels.hidden', true) . ']</em>';
+			}
 			$outHeader=  ($row['date'] ? htmlspecialchars($this->itemLabels['date'].' '.t3lib_BEfunc::date($row['date'])).'<br />':'').
 					$this->infoGif($infoArr).
-					'<b>'.$this->linkEditContent($this->renderText($row['header']),$row).'</b><br />';
+					'<b>' . $this->linkEditContent($this->renderText($row['header']), $row) . $hiddenHeaderNote . '</b><br />';
 		}
 
 			// Make content:
@@ -1987,9 +1990,9 @@ class tx_cms_layout extends recordList {
 				// Remove disallowed languages
 			if (count($langSelItems) > 1 &&
 				!$GLOBALS['BE_USER']->user['admin'] &&
-				strlen($GLOBALS['BE_USER']->dataLists['allowed_languages'])) {
+				strlen($GLOBALS['BE_USER']->groupData['allowed_languages'])) {
 
-				$allowed_languages = array_flip(explode(',', $GLOBALS['BE_USER']->dataLists['allowed_languages']));
+				$allowed_languages = array_flip(explode(',', $GLOBALS['BE_USER']->groupData['allowed_languages']));
 
 				if (count($allowed_languages)) {
 					foreach($langSelItems as $key => $value) {
