@@ -2,7 +2,7 @@
 /***************************************************************
 *  Copyright notice
 *
-*  (c) 1999-2005 Kasper Skaarhoj (kasperYYYY@typo3.com)
+*  (c) 1999-2008 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
@@ -28,7 +28,7 @@
  * New content elements wizard
  * (Part of the 'cms' extension)
  *
- * $Id: db_new_content_el.php 1421 2006-04-10 09:27:15Z mundaun $
+ * $Id: db_new_content_el.php 3775 2008-06-10 12:05:31Z patrick $
  * Revised for TYPO3 3.6 November/2003 by Kasper Skaarhoj
  * XHTML compatible.
  *
@@ -161,7 +161,13 @@ class SC_db_new_content_el {
 
 		// Internal, static:
 	var $modTSconfig=array();	// Module TSconfig.
-	var $doc;					// Internal backend template object
+
+	/**
+	 * Internal backend template object
+	 *
+	 * @var mediumDoc
+	 */
+	var $doc;
 
 		// Internal, dynamic:
 	var $include_once = array();	// Includes a list of files to include between init() and main() - see init()
@@ -192,16 +198,20 @@ class SC_db_new_content_el {
 		$this->modTSconfig = t3lib_BEfunc::getModTSconfig($this->id,'mod.'.$this->MCONF['name']);
 
 			// Starting the document template object:
-		$this->doc = t3lib_div::makeInstance('mediumDoc');
+		$this->doc = t3lib_div::makeInstance('template');
 		$this->doc->docType= 'xhtml_trans';
 		$this->doc->backPath = $BACK_PATH;
+		$this->doc->setModuleTemplate('templates/db_new_content_el.html');
 		$this->doc->JScode='';
 		$this->doc->form='<form action="" name="editForm"><input type="hidden" name="defValues" value="" />';
 
+			// Setting up the context sensitive menu:
+		$this->doc->getContextMenuCode();
+
 			// Getting the current page and receiving access information (used in main())
 		$perms_clause = $BE_USER->getPagePermsClause(1);
-		$pageinfo = t3lib_BEfunc::readPageAccess($this->id,$perms_clause);
-		$this->access = is_array($pageinfo) ? 1 : 0;
+		$this->pageinfo = t3lib_BEfunc::readPageAccess($this->id,$perms_clause);
+		$this->access = is_array($this->pageinfo) ? 1 : 0;
 	}
 
 	/**
@@ -242,17 +252,9 @@ class SC_db_new_content_el {
 			// Creating content
 			// ***************************
 			$this->content='';
-			$this->content.=$this->doc->startPage($LANG->getLL('newContentElement'));
 			$this->content.=$this->doc->header($LANG->getLL('newContentElement'));
 			$this->content.=$this->doc->spacer(5);
 
-			$elRow = t3lib_BEfunc::getRecordWSOL('pages',$this->id);
-			$hline = t3lib_iconWorks::getIconImage('pages',$elRow,$BACK_PATH,' title="'.htmlspecialchars(t3lib_BEfunc::getRecordIconAltText($elRow,'pages')).'" align="top"');
-			$hline.= t3lib_BEfunc::getRecordTitle('pages',$elRow,1);
-			$this->content.=$this->doc->section('',$hline,0,1);
-			$this->content.=$this->doc->spacer(10);
-
-
 				// Wizard
 			$code='';
 			$lines=array();
@@ -261,7 +263,7 @@ class SC_db_new_content_el {
 				// Traverse items for the wizard.
 				// An item is either a header or an item rendered with a radio button and title/description and icon:
 			$cc=0;
-			foreach($wizardItems as $k => $wInfo)	{
+			foreach ($wizardItems as $k => $wInfo)	{
 				if ($wInfo['header'])	{
 					if ($cc>0) $lines[]='
 						<tr>
@@ -279,7 +281,7 @@ class SC_db_new_content_el {
 					$tL[]='<input type="radio" name="tempB" value="'.htmlspecialchars($k).'" onclick="'.htmlspecialchars($this->doc->thisBlur().$oC).'" />';
 
 						// Onclick action for icon/title:
-					$aOnClick = 'document.editForm.tempB['.$cc.'].checked=1;'.$this->doc->thisBlur().$oC.'return false;';
+					$aOnClick = 'document.getElementsByName(\'tempB\')['.$cc.'].checked=1;'.$this->doc->thisBlur().$oC.'return false;';
 
 						// Icon:
 					$iInfo = @getimagesize($wInfo['icon']);
@@ -307,7 +309,7 @@ class SC_db_new_content_el {
 				<table border="0" cellpadding="1" cellspacing="2" id="typo3-ceWizardTable">
 					'.implode('',$lines).'
 				</table>';
-			$this->content.=$this->doc->section(!$onClickEvent?$LANG->getLL('1_selectType'):'',$code,0,1);
+			$this->content.= $this->doc->section(!$onClickEvent?$LANG->getLL('1_selectType'):'',$code,0,1);
 
 
 
@@ -315,11 +317,11 @@ class SC_db_new_content_el {
 			if (!$onClickEvent)	{
 
 					// Add anchor "sel2"
-				$this->content.=$this->doc->section('','<a name="sel2"></a>');
-				$this->content.=$this->doc->spacer(20);
+				$this->content.= $this->doc->section('','<a name="sel2"></a>');
+				$this->content.= $this->doc->spacer(20);
 
 					// Select position
-				$code=$LANG->getLL('sel2',1).'<br /><br />';
+				$code = $LANG->getLL('sel2',1).'<br /><br />';
 
 					// Load SHARED page-TSconfig settings and retrieve column list from there, if applicable:
 				$modTSconfig_SHARED = t3lib_BEfunc::getModTSconfig($this->id,'mod.SHARED');
@@ -327,28 +329,25 @@ class SC_db_new_content_el {
 				$colPosList = implode(',',array_unique(t3lib_div::intExplode(',',$colPosList)));		// Removing duplicates, if any
 
 					// Finally, add the content of the column selector to the content:
-				$code.=$posMap->printContentElementColumns($this->id,0,$colPosList,1,$this->R_URI);
-				$this->content.=$this->doc->section($LANG->getLL('2_selectPosition'),$code,0,1);
+				$code.= $posMap->printContentElementColumns($this->id,0,$colPosList,1,$this->R_URI);
+				$this->content.= $this->doc->section($LANG->getLL('2_selectPosition'),$code,0,1);
 			}
-
-				// IF there is a return-url set, then print a go-back link:
-			if ($this->R_URI)	{
-				$code='<br /><br /><a href="'.htmlspecialchars($this->R_URI).'" class="typo3-goBack"><img'.t3lib_iconWorks::skinImg($this->doc->backPath,'gfx/goback.gif','width="14" height="14"').' alt="" />'.$LANG->getLL('goBack',1).'</a>';
-				$this->content.=$this->doc->section('',$code,0,1);
-			}
-
-				// Add CSH:
-			$this->content.= $this->doc->section('',t3lib_BEfunc::cshItem('xMOD_csh_corebe', 'new_ce', $GLOBALS['BACK_PATH'],'<br/>'),0,1);
-
-				// Add a very high clear-gif, 700 px (so that the link to the anchor "sel2" shows this part in top for sure...)
-			$this->content.=$this->doc->section('','<img src="clear.gif" width="1" height="700" alt="" />',0,1);
-
 		} else {		// In case of no access:
-			$this->content='';
-			$this->content.=$this->doc->startPage($LANG->getLL('newContentElement'));
-			$this->content.=$this->doc->header($LANG->getLL('newContentElement'));
-			$this->content.=$this->doc->spacer(5);
+			$this->content = '';
+			$this->content.= $this->doc->header($LANG->getLL('newContentElement'));
+			$this->content.= $this->doc->spacer(5);
 		}
+
+			// Setting up the buttons and markers for docheader
+		$docHeaderButtons = $this->getButtons();
+		$markers['CSH'] = $docHeaderButtons['csh'];
+		$markers['CONTENT'] = $this->content;
+
+			// Build the <body> for the module
+		$this->content = $this->doc->startPage($LANG->getLL('newContentElement'));
+		$this->content.= $this->doc->moduleBody($this->pageinfo, $docHeaderButtons, $markers);
+		$this->content.= $this->doc->endPage();
+		$this->content = $this->doc->insertStylesAndJS($this->content);
 	}
 
 	/**
@@ -357,13 +356,38 @@ class SC_db_new_content_el {
 	 * @return	void
 	 */
 	function printContent()	{
-		global $SOBE;
-
-		$this->content.= $this->doc->endPage();
 		echo $this->content;
 	}
 
+	/**
+	 * Create the panel of buttons for submitting the form or otherwise perform operations.
+	 *
+	 * @return	array	all available buttons as an assoc. array
+	 */
+	protected function getButtons()	{
+		global $LANG, $BACK_PATH;
 
+		$buttons = array(
+			'csh' => '',
+			'back' => ''
+		);
+
+
+		if ($this->id && $this->access)	{
+				// CSH
+			$buttons['csh'] = t3lib_BEfunc::cshItem('xMOD_csh_corebe', 'new_ce', $GLOBALS['BACK_PATH'], '', TRUE);
+
+				// Back
+			if ($this->R_URI)	{
+				$buttons['back'] = '<a href="' . htmlspecialchars($this->R_URI) . '" class="typo3-goBack">' .
+					'<img' . t3lib_iconWorks::skinImg($this->doc->backPath, 'gfx/goback.gif') . ' alt="" title="' . $LANG->getLL('goBack', 1) . '" />' .
+					'</a>';
+			}
+		}
+
+
+		return $buttons;
+	}
 
 
 
@@ -556,7 +580,7 @@ Contact me: | tv=check | 1
 		$row = array('pid'=>$this->id);
 		$TCEFORM_TSconfig = t3lib_BEfunc::getTCEFORM_TSconfig('tt_content',$row);
 		$removeItems = t3lib_div::trimExplode(',',$TCEFORM_TSconfig['CType']['removeItems'],1);
-		
+
 		$headersUsed = Array();
 			// Traverse wizard items:
 		foreach($wizardItems as $key => $cfg)	{
@@ -583,7 +607,7 @@ Contact me: | tv=check | 1
 						$config = &$TCA['tt_content']['columns'][$fN]['config'];
 						$authModeDeny = $config['type']=='select' && $config['authMode'] && !$GLOBALS['BE_USER']->checkAuthMode('tt_content',$fN,$fV,$config['authMode']);
 
-						if ($authModeDeny || in_array($fV,$removeItems))	{
+						if ($authModeDeny || ($fN=='CType' && in_array($fV,$removeItems)))	{
 								// Remove element all together:
 							unset($wizardItems[$key]);
 							break;
