@@ -2,7 +2,7 @@
 /***************************************************************
 *  Copyright notice
 *
-*  (c) 1999-2004 Kasper Skaarhoj (kasperYYYY@typo3.com)
+*  (c) 1999-2008 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
@@ -27,7 +27,7 @@
 /**
  * Dynamic configuation of the system-related tables, typ. sys_* series
  *
- * $Id: tbl_cms.php,v 1.12 2004/09/13 22:57:35 typo3 Exp $
+ * $Id: tbl_cms.php 3980 2008-08-15 17:18:45Z benni $
  *
  * @author	Kasper Skaarhoj <kasperYYYY@typo3.com>
  */
@@ -44,7 +44,7 @@
 $TCA['fe_users'] = Array (
 	'ctrl' => $TCA['fe_users']['ctrl'],
 	'interface' => Array (
-		'showRecordFieldList' => 'username,password,usergroup,lockToDomain,name,title,address,zip,city,country,email,www,telephone,fax,disable,starttime,endtime'
+		'showRecordFieldList' => 'username,password,usergroup,lockToDomain,name,title,company,address,zip,city,country,email,www,telephone,fax,disable,starttime,endtime'
 	),
 	'feInterface' => $TCA['fe_users']['feInterface'],
 	'columns' => Array (
@@ -63,7 +63,7 @@ $TCA['fe_users'] = Array (
 				'type' => 'input',
 				'size' => '10',
 				'max' => '40',
-				'eval' => 'nospace,lower,required'
+				'eval' => 'nospace,required,password'
 			)
 		),
 		'usergroup' => Array (
@@ -71,7 +71,7 @@ $TCA['fe_users'] = Array (
 			'config' => Array (
 				'type' => 'select',
 				'foreign_table' => 'fe_groups',
-				'size' => '3',
+				'size' => '6',
 				'minitems' => '1',
 				'maxitems' => '50'
 			)
@@ -84,7 +84,8 @@ $TCA['fe_users'] = Array (
 				'size' => '20',
 				'eval' => 'trim',
 				'max' => '50',
-				'checkbox' => ''
+				'checkbox' => '',
+				'softref' => 'substitute'
 			)
 		),
 		'name' => Array (
@@ -131,14 +132,14 @@ $TCA['fe_users'] = Array (
 			'label' => 'LLL:EXT:lang/locallang_general.php:LGL.email',
 			'config' => Array (
 				'type' => 'input',
-				'size' => '40',
+				'size' => '20',
 				'eval' => 'trim',
 				'max' => '80'
 			)
 		),
 		'title' => Array (
 			'exclude' => 1,
-			'label' => 'LLL:EXT:lang/locallang_general.php:LGL.title',
+			'label' => 'LLL:EXT:lang/locallang_general.php:LGL.title_person',
 			'config' => Array (
 				'type' => 'input',
 				'size' => '20',
@@ -203,7 +204,7 @@ $TCA['fe_users'] = Array (
 				'type' => 'group',
 				'internal_type' => 'file',
 				'allowed' => $GLOBALS['TYPO3_CONF_VARS']['GFX']['imagefile_ext'],
-				'max_size' => '1000',
+				'max_size' => $GLOBALS['TYPO3_CONF_VARS']['BE']['maxFileSize'],
 				'uploadfolder' => 'uploads/pics',
 				'show_thumbs' => '1',
 				'size' => '3',
@@ -242,7 +243,6 @@ $TCA['fe_users'] = Array (
 				'default' => '0',
 				'range' => Array (
 					'upper' => mktime(0,0,0,12,31,2020),
-					'lower' => mktime(0,0,0,date('m')-1,date('d'),date('Y'))
 				)
 			)
 		),
@@ -252,7 +252,7 @@ $TCA['fe_users'] = Array (
 			'config' => Array (
 				'type' => 'text',
 				'cols' => '40',
-				'rows' => '5',
+				'rows' => '10',
 				'wizards' => Array(
 					'_PADDING' => 4,
 					'0' => Array(
@@ -262,16 +262,24 @@ $TCA['fe_users'] = Array (
 						'icon' => 'wizard_tsconfig.gif',
 						'JSopenParams' => 'height=500,width=780,status=0,menubar=0,scrollbars=1',
 					)
-				)
-			)
+				),
+				'softref' => 'TSconfig'
+			),
+			'defaultExtras' => 'fixed-font : enable-tab',
 		)
 	),
 	'types' => Array (
-		'0' => Array('showitem' => 'username;;;;2-2-2, password, usergroup, lockToDomain, --div--, name;;2;;3-3-3, address, zip, city, country, telephone, fax, email, www, image;;;;4-4-4, --div--, TSconfig;;;;5-5-5')
+		'0' => Array('showitem' => '
+			disable,username;;;;1-1-1, password, usergroup,
+			--div--;LLL:EXT:cms/locallang_tca.xml:fe_users.tabs.personelData, name;;1;;1-1-1, address, zip, city, country, telephone, fax, email, www, image;;;;2-2-2,
+			--div--;LLL:EXT:cms/locallang_tca.xml:fe_users.tabs.options, lockToDomain;;;;1-1-1, TSconfig;;;;2-2-2,
+			--div--;LLL:EXT:cms/locallang_tca.xml:fe_users.tabs.access, starttime, endtime,
+			--div--;LLL:EXT:cms/locallang_tca.xml:fe_users.tabs.extended
+
+		')
 	),
 	'palettes' => Array (
-		'1' => Array('showitem' => 'disable, starttime, endtime'),
-		'2' => Array('showitem' => 'title,company')
+		'1' => Array('showitem' => 'title,company')
 	)
 );
 
@@ -287,7 +295,7 @@ $TCA['fe_users'] = Array (
 $TCA['fe_groups'] = Array (
 	'ctrl' => $TCA['fe_groups']['ctrl'],
 	'interface' => Array (
-		'showRecordFieldList' => 'title,hidden,lockToDomain,description'
+		'showRecordFieldList' => 'title,hidden,subgroup,lockToDomain,description'
 	),
 	'columns' => Array (
 		'hidden' => Array (
@@ -303,10 +311,23 @@ $TCA['fe_groups'] = Array (
 			'config' => Array (
 				'type' => 'input',
 				'size' => '20',
-				'max' => '20',
+				'max' => '50',
 				'eval' => 'trim,required'
 			)
 		),
+		'subgroup' => Array (
+			'exclude' => 1,
+			'label' => 'LLL:EXT:cms/locallang_tca.php:fe_groups.subgroup',
+			'config' => Array (
+				'type' => 'select',
+				'foreign_table' => 'fe_groups',
+				'foreign_table_where' => 'AND NOT(fe_groups.uid = ###THIS_UID###) AND fe_groups.hidden=0 ORDER BY fe_groups.title',
+				'size' => 6,
+				'autoSizeMax' => 10,
+				'minitems' => 0,
+				'maxitems' => 20
+			)
+		),
 		'lockToDomain' => Array (
 			'exclude' => 1,
 			'label' => 'LLL:EXT:cms/locallang_tca.php:fe_groups.lockToDomain',
@@ -332,7 +353,7 @@ $TCA['fe_groups'] = Array (
 			'config' => Array (
 				'type' => 'text',
 				'cols' => '40',
-				'rows' => '5',
+				'rows' => '10',
 				'wizards' => Array(
 					'_PADDING' => 4,
 					'0' => Array(
@@ -342,12 +363,18 @@ $TCA['fe_groups'] = Array (
 						'icon' => 'wizard_tsconfig.gif',
 						'JSopenParams' => 'height=500,width=780,status=0,menubar=0,scrollbars=1',
 					)
-				)
-			)
+				),
+				'softref' => 'TSconfig'
+			),
+			'defaultExtras' => 'fixed-font : enable-tab',
 		)
 	),
 	'types' => Array (
-		'0' => Array('showitem' => 'hidden;;;;1-1-1,title;;;;2-2-2,lockToDomain,description;;;;3-3-3, --div--, TSconfig;;;;5-5-5')
+		'0' => Array('showitem' => '
+			hidden;;;;1-1-1,title;;;;2-2-2,description,subgroup;;;;3-3-3,
+			--div--;LLL:EXT:cms/locallang_tca.xml:fe_groups.tabs.options, lockToDomain;;;;1-1-1, TSconfig;;;;2-2-2,
+			--div--;LLL:EXT:cms/locallang_tca.xml:fe_groups.tabs.extended
+		')
 	)
 );
 
@@ -369,8 +396,9 @@ $TCA['sys_domain'] = Array (
 				'type' => 'input',
 				'size' => '35',
 				'max' => '80',
-				'eval' => 'required,unique,lower,trim'
-			)
+				'eval' => 'required,unique,lower,trim',
+				'softref' => 'substitute'
+			),
 		),
 		'redirectTo' => Array (
 			'label' => 'LLL:EXT:cms/locallang_tca.php:sys_domain.redirectTo',
@@ -380,8 +408,9 @@ $TCA['sys_domain'] = Array (
 				'max' => '120',
 				'checkbox' => '',
 				'default' => '',
-				'eval' => 'trim'
-			)
+				'eval' => 'trim',
+				'softref' => 'substitute'
+			),
 		),
 		'hidden' => Array (
 			'label' => 'LLL:EXT:lang/locallang_general.php:LGL.disable',
@@ -426,7 +455,7 @@ $TCA['pages_language_overlay'] = Array (
 			'label' => 'LLL:EXT:lang/locallang_general.php:LGL.hidden',
 			'config' => Array (
 				'type' => 'check',
-				'default' => '1'
+				'default' => '0'
 			)
 		),
 		'starttime' => Array (
@@ -453,12 +482,13 @@ $TCA['pages_language_overlay'] = Array (
 				'default' => '0',
 				'range' => Array (
 					'upper' => mktime(0,0,0,12,31,2020),
-					'lower' => mktime(0,0,0,date('m')-1,date('d'),date('Y'))
 				)
 			)
 		),
 		'title' => Array (
+			'l10n_mode' => 'prefixLangTitle',
 			'label' => $TCA['pages']['columns']['title']['label'],
+			'l10n_cat' => 'text',
 			'config' => Array (
 				'type' => 'input',
 				'size' => '30',
@@ -468,6 +498,7 @@ $TCA['pages_language_overlay'] = Array (
 		),
 		'subtitle' => Array (
 			'exclude' => 1,
+			'l10n_cat' => 'text',
 			'label' => $TCA['pages']['columns']['subtitle']['label'],
 			'config' => Array (
 				'type' => 'input',
@@ -478,6 +509,7 @@ $TCA['pages_language_overlay'] = Array (
 		),
 		'nav_title' => Array (
 			'exclude' => 1,
+			'l10n_cat' => 'text',
 			'label' => $TCA['pages']['columns']['nav_title']['label'],
 			'config' => Array (
 				'type' => 'input',
@@ -526,7 +558,7 @@ $TCA['pages_language_overlay'] = Array (
 		),
 		'author_email' => Array (
 			'exclude' => 1,
-			'label' => $TCA['pages']['columns']['author']['label'],
+			'label' => $TCA['pages']['columns']['author_email']['label'],
 			'config' => Array (
 				'type' => 'input',
 				'size' => '20',
@@ -541,7 +573,7 @@ $TCA['pages_language_overlay'] = Array (
 				'type' => 'group',
 				'internal_type' => 'file',
 				'allowed' => $TCA['pages']['columns']['media']['config']['allowed'],
-				'max_size' => '2000',
+				'max_size' => $GLOBALS['TYPO3_CONF_VARS']['BE']['maxFileSize'],
 				'uploadfolder' => 'uploads/media',
 				'show_thumbs' => '1',
 				'size' => '3',
@@ -592,7 +624,7 @@ $TCA['sys_template'] = Array (
 	),
 	'columns' => Array (
 		'title' => Array (
-			'label' => 'Template title:',
+			'label' => 'LLL:EXT:cms/locallang_tca.xml:sys_template.title',
 			'config' => Array (
 				'type' => 'input',
 				'size' => '25',
@@ -601,7 +633,7 @@ $TCA['sys_template'] = Array (
 			)
 		),
 		'hidden' => Array (
-			'label' => 'Deactivated:',
+			'label' => 'LLL:EXT:lang/locallang_general.php:LGL.disable',
 			'exclude' => 1,
 			'config' => Array (
 				'type' => 'check',
@@ -609,7 +641,7 @@ $TCA['sys_template'] = Array (
 			)
 		),
 		'starttime' => Array (
-			'label' => 'Start:',
+			'label' => 'LLL:EXT:lang/locallang_general.php:LGL.starttime',
 			'exclude' => 1,
 			'config' => Array (
 				'type' => 'input',
@@ -621,7 +653,7 @@ $TCA['sys_template'] = Array (
 			)
 		),
 		'endtime' => Array (
-			'label' => 'Stop:',
+			'label' => 'LLL:EXT:lang/locallang_general.php:LGL.endtime',
 			'exclude' => 1,
 			'config' => Array (
 				'type' => 'input',
@@ -632,18 +664,17 @@ $TCA['sys_template'] = Array (
 				'default' => '0',
 				'range' => Array (
 					'upper' => mktime(0,0,0,12,31,2020),
-					'lower' => mktime(0,0,0,date('m')-1,date('d'),date('Y'))
 				)
 			)
 		),
 		'root' => Array (
-			'label' => 'Rootlevel:',
+			'label' => 'LLL:EXT:cms/locallang_tca.xml:sys_template.root',
 			'config' => Array (
 				'type' => 'check'
 			)
 		),
 		'clear' => Array (
-			'label' => 'Clear:',
+			'label' => 'LLL:EXT:cms/locallang_tca.xml:sys_template.clear',
 			'config' => Array (
 				'type' => 'check',
 				'items' => Array (
@@ -654,7 +685,7 @@ $TCA['sys_template'] = Array (
 			)
 		),
 		'sitetitle' => Array (
-			'label' => 'Website title:',
+			'label' => 'LLL:EXT:cms/locallang_tca.xml:sys_template.sitetitle',
 			'config' => Array (
 				'type' => 'input',
 				'size' => '25',
@@ -662,21 +693,23 @@ $TCA['sys_template'] = Array (
 			)
 		),
 		'constants' => Array (
-			'label' => 'Constants:',
+			'label' => 'LLL:EXT:cms/locallang_tca.xml:sys_template.constants',
 			'config' => Array (
 				'type' => 'text',
 				'cols' => '48',
 				'rows' => '10',
-				'wrap' => 'OFF'
-			)
+				'wrap' => 'OFF',
+				'softref' => 'TStemplate,email[subst],url[subst]'
+			),
+			'defaultExtras' => 'fixed-font : enable-tab',
 		),
 		'resources' => Array (
-			'label' => 'Resources:',
+			'label' => 'LLL:EXT:cms/locallang_tca.xml:sys_template.resources',
 			'config' => Array (
 				'type' => 'group',
 				'internal_type' => 'file',
-				'allowed' => $GLOBALS['TYPO3_CONF_VARS']['GFX']['imagefile_ext'].',html,htm,ttf,pfb,pfm,txt,css,tmpl,inc,ico',
-				'max_size' => '1000',
+				'allowed' => $GLOBALS['TYPO3_CONF_VARS']['GFX']['imagefile_ext'].',html,htm,ttf,pfb,pfm,txt,css,tmpl,inc,ico,js,xml',
+				'max_size' => $GLOBALS['TYPO3_CONF_VARS']['BE']['maxFileSize'],
 				'uploadfolder' => 'uploads/tf',
 				'show_thumbs' => '1',
 				'size' => '7',
@@ -685,41 +718,42 @@ $TCA['sys_template'] = Array (
 			)
 		),
 		'nextLevel' => Array (
-			'label' => 'Template on next level:',
+			'label' => 'LLL:EXT:cms/locallang_tca.xml:sys_template.nextLevel',
 			'config' => Array (
 				'type' => 'group',
 				'internal_type' => 'db',
 				'allowed' => 'sys_template',
 				'show_thumbs' => '1',
-				'size' => '3',
+				'size' => '1',
 				'maxitems' => '1',
 				'minitems' => '0',
 				'default' => ''
 			)
 		),
 		'include_static' => Array (
-			'label' => 'Include static:',
+			'label' => 'LLL:EXT:cms/locallang_tca.xml:sys_template.include_static',
 			'config' => Array (
 				'type' => 'select',
 				'foreign_table' => 'static_template',
 				'foreign_table_where' => 'ORDER BY static_template.title DESC',
 				'size' => 10,
 				'maxitems' => 20,
-				'default' => ''
-			)
+				'default' => '',
+			),
 		),
 		'include_static_file' => Array(
-			'label' => 'Include static (from extensions):',
+			'label' => 'LLL:EXT:cms/locallang_tca.xml:sys_template.include_static_file',
 			'config' => Array (
 				'type' => 'select',
 				'size' => 10,
-				'maxitems' => 20,
+				'maxitems' => 100,
 				'items' => Array (
-				)
+				),
+				'softref' => 'ext_fileref'
 			)
 		),
 		'basedOn' => Array (
-			'label' => 'Include basis template:',
+			'label' => 'LLL:EXT:cms/locallang_tca.xml:sys_template.basedOn',
 			'config' => Array (
 				'type' => 'group',
 				'internal_type' => 'db',
@@ -735,7 +769,7 @@ $TCA['sys_template'] = Array (
 					'_VERTICAL' => 1,
 					'edit' => Array(
 						'type' => 'popup',
-						'title' => 'Edit filemount',
+						'title' => 'Edit template',
 						'script' => 'wizard_edit.php',
 						'popup_onlyOpenIfSelected' => 1,
 						'icon' => 'edit2.gif',
@@ -743,7 +777,7 @@ $TCA['sys_template'] = Array (
 					),
 					'add' => Array(
 						'type' => 'script',
-						'title' => 'Add new basis template',
+						'title' => 'LLL:EXT:cms/locallang_tca.xml:sys_template.basedOn_add',
 						'icon' => 'add.gif',
 						'params' => Array(
 							'table'=>'sys_template',
@@ -756,7 +790,7 @@ $TCA['sys_template'] = Array (
 			)
 		),
 		'includeStaticAfterBasedOn' => Array (
-			'label' => 'Include static AFTER basedOn:',
+			'label' => 'LLL:EXT:cms/locallang_tca.xml:sys_template.includeStaticAfterBasedOn',
 			'exclude' => 1,
 			'config' => Array (
 				'type' => 'check',
@@ -764,7 +798,7 @@ $TCA['sys_template'] = Array (
 			)
 		),
 		'config' => Array (
-			'label' => 'Setup:',
+			'label' => 'LLL:EXT:cms/locallang_tca.xml:sys_template.config',
 			'config' => Array (
 				'type' => 'text',
 				'rows' => 10,
@@ -779,28 +813,31 @@ $TCA['sys_template'] = Array (
 						'JSopenParams' => 'height=500,width=780,status=0,menubar=0,scrollbars=1',
 					)
 				),
-				'wrap' => 'OFF'
-			)
+				'wrap' => 'OFF',
+				'softref' => 'TStemplate,email[subst],url[subst]'
+			),
+			'defaultExtras' => 'fixed-font : enable-tab',
 		),
 		'editorcfg' => Array (
-			'label' => 'Backend Editor Configuration:',
+			'label' => 'LLL:EXT:cms/locallang_tca.xml:sys_template.editorcfg',
 			'config' => Array (
 				'type' => 'text',
-				'rows' => 4,
+				'rows' => 8,
 				'cols' => 48,
 				'wrap' => 'OFF'
-			)
+			),
+			'defaultExtras' => 'fixed-font : enable-tab',
 		),
 		'description' => Array (
-			'label' => 'Description:',
+			'label' => 'LLL:EXT:cms/locallang_tca.xml:sys_template.description',
 			'config' => Array (
 				'type' => 'text',
-				'rows' => 10,
+				'rows' => 5,
 				'cols' => 48
 			)
 		),
 		'static_file_mode' => Array (
-			'label' => 'Static template files from T3 Extensions:',
+			'label' => 'LLL:EXT:cms/locallang_tca.xml:sys_template.static_file_mode',
 			'config' => Array (
 				'type' => 'select',
 				'items' => Array (
@@ -822,11 +859,13 @@ $TCA['sys_template'] = Array (
 		),
 	),
 	'types' => Array (
-		'1' => Array('showitem' => 'title;;1;;2-2-2, sitetitle, constants;;;;3-3-3, config, resources, clear, root, --div--, include_static;;2;;5-5-5, include_static_file;;2, basedOn;;2, static_file_mode, nextLevel, --div--, description;;;;5-5-5, editorcfg')
-	),
-	'palettes' => Array (
-		'1' => Array('showitem' => 'hidden,starttime,endtime'),
-		'2' => Array('showitem' => 'includeStaticAfterBasedOn')
+		'1' => Array('showitem' => '
+			hidden,title;;1;;2-2-2, sitetitle, constants;;;;3-3-3, config, description;;;;4-4-4,
+			--div--;LLL:EXT:cms/locallang_tca.xml:sys_template.tabs.options, clear, root, nextLevel, editorcfg;;;;5-5-5,
+			--div--;LLL:EXT:cms/locallang_tca.xml:sys_template.tabs.include, include_static,includeStaticAfterBasedOn,6-6-6, include_static_file, basedOn, static_file_mode,
+			--div--;LLL:EXT:cms/locallang_tca.xml:sys_template.tabs.files, resources,
+			--div--;LLL:EXT:cms/locallang_tca.xml:sys_template.tabs.access, starttime, endtime'
+		)
 	)
 );
 
@@ -859,7 +898,8 @@ $TCA['static_template'] = Array (
 				'cols' => '48',
 				'rows' => '10',
 				'wrap' => 'OFF'
-			)
+			),
+			'defaultExtras' => 'fixed-font : enable-tab',
 		),
 		'include_static' => Array (
 			'label' => 'Include static:',
@@ -879,7 +919,8 @@ $TCA['static_template'] = Array (
 				'rows' => 10,
 				'cols' => 48,
 				'wrap' => 'OFF'
-			)
+			),
+			'defaultExtras' => 'fixed-font : enable-tab',
 		),
 		'editorcfg' => Array (
 			'label' => 'Backend Editor Configuration:',
@@ -888,7 +929,8 @@ $TCA['static_template'] = Array (
 				'rows' => 4,
 				'cols' => 48,
 				'wrap' => 'OFF'
-			)
+			),
+			'defaultExtras' => 'fixed-font : enable-tab',
 		),
 		'description' => Array (
 			'label' => 'Description:',
