@@ -2,7 +2,7 @@
 /***************************************************************
 *  Copyright notice
 *
-*  (c) 1999-2004 Kasper Skaarhoj (kasperYYYY@typo3.com)
+*  (c) 1999-2008 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
@@ -28,7 +28,7 @@
  * Generating gif/png-files from TypoScript
  * Used by the menu-objects and imgResource in TypoScript.
  *
- * $Id: class.tslib_gifbuilder.php,v 1.9 2004/09/13 22:57:36 typo3 Exp $
+ * $Id: class.tslib_gifbuilder.php 3439 2008-03-16 19:16:51Z flyguide $
  * Revised for TYPO3 3.6 June/2003 by Kasper Skaarhoj
  *
  * @author	Kasper Skaarhoj <kasperYYYY@typo3.com>
@@ -39,17 +39,17 @@
  *
  *
  *  102: class tslib_gifBuilder extends t3lib_stdGraphic
- *  127:     function start($conf,$data)
- *  296:     function gifBuild()
- *  324:     function make()
+ *  129:     function start($conf,$data)
+ *  315:     function gifBuild()
+ *  343:     function make()
  *
  *              SECTION: Various helper functions
- *  475:     function checkTextObj($conf)
- *  552:     function calcOffset($string)
- *  601:     function getResource($file,$fileArray)
- *  616:     function checkFile($file)
- *  627:     function fileName($pre)
- *  637:     function extension()
+ *  486:     function checkTextObj($conf)
+ *  566:     function calcOffset($string)
+ *  615:     function getResource($file,$fileArray)
+ *  632:     function checkFile($file)
+ *  643:     function fileName($pre)
+ *  659:     function extension()
  *
  * TOTAL FUNCTIONS: 9
  * (This index is automatically created/updated by the extension "extdeveval")
@@ -108,6 +108,8 @@ class tslib_gifBuilder extends t3lib_stdGraphic {
 	var $map;			// map-data
 	var $workArea;
 	var $setup = Array ();		// This holds the operational setup for gifbuilder. Basically this is a TypoScript array with properties.
+	var $combinedTextStrings = array();		// Contains all text strings used on this image
+	var $combinedFileNames = array();		// Contains all filenames (basename without extension) used on this image
 	var $data = Array();		// This is the array from which data->field: [key] is fetched. So this is the current record!
 	var $objBB = Array();
 	var $myClassName = 'gifbuilder';
@@ -130,6 +132,22 @@ class tslib_gifBuilder extends t3lib_stdGraphic {
 			$this->setup = $conf;
 			$this->data = $data;
 
+
+			/* Hook preprocess gifbuilder conf
+			 * Added by Julle for 3.8.0
+			 *
+			 * Let's you pre-process the gifbuilder configuration. for
+			 * example you can split a string up into lines and render each
+			 * line as TEXT obj, see extension julle_gifbconf
+			 */
+
+			if (is_array($GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['tslib/class.tslib_gifbuilder.php']['gifbuilder-ConfPreProcess']))    {
+				foreach($GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['tslib/class.tslib_gifbuilder.php']['gifbuilder-ConfPreProcess'] as $_funcRef)    {
+					$_params = $this->setup;
+					$this->setup = t3lib_div::callUserFunction($_funcRef,$_params,$this);
+				}
+			}
+
 				// Initializing global Char Range Map
 			$this->charRangeMap = array();
 			if (is_array($GLOBALS['TSFE']->tmpl->setup['_GIFBUILDER.']['charRangeMap.']))	{
@@ -156,17 +174,20 @@ class tslib_gifBuilder extends t3lib_stdGraphic {
 				$cObj->start($this->data);
 				$this->setup['backColor'] = trim($cObj->stdWrap($this->setup['backColor'], $this->setup['backColor.']));
 			}
-			if (!$this->setup['backColor'])	{$this->setup['backColor']='white';}
+			if (!$this->setup['backColor'])	{ $this->setup['backColor']='white'; }
 
-				// Transparent GIFs
-				// not working with reduceColors
-				// there's an option for IM: -transparent colors
 			if ($conf['transparentColor.'] || $conf['transparentColor'])	{
 				$cObj =t3lib_div::makeInstance('tslib_cObj');
 				$cObj->start($this->data);
 				$this->setup['transparentColor_array'] = explode('|', trim($cObj->stdWrap($this->setup['transparentColor'], $this->setup['transparentColor.'])));
 			}
 
+ 				// Transparency does not properly work when, GIFs or 8-bit PNGs are generated or reduceColors is set -- disable truecolor flag so they get generated "natively" in 8-bit.
+ 				// not working with reduceColors and truecolor images
+ 			if (($this->setup['transparentBackground'] || is_array($this->setup['transparentColor_array'])) && ($this->gifExtension=='gif' || !$this->png_truecolor || isset($this->setup['reduceColors'])))	{
+ 				$this->truecolor = false;
+ 			}
+
 				// Set default dimensions
 			if (!$this->setup['XY'])	{$this->setup['XY']='120,50';}
 
@@ -174,7 +195,7 @@ class tslib_gifBuilder extends t3lib_stdGraphic {
 				// Checking TEXT and IMAGE objects for files. If any errors the objects are cleared.
 				// The Bounding Box for the objects is stored in an array
 			foreach($sKeyArray as $theKey) {
-				$theValue=$this->setup[$theKey];
+				$theValue = $this->setup[$theKey];
 
 				if (intval($theKey) && $conf=$this->setup[$theKey.'.'])	{
 						// Swipes through TEXT and IMAGE-objects
@@ -189,21 +210,22 @@ class tslib_gifBuilder extends t3lib_stdGraphic {
 
 									// Calculate bounding box:
 								$txtInfo=$this->calcBBox($this->setup[$theKey.'.']);
-								$this->setup[$theKey.'.']['BBOX']=$txtInfo;
-								$this->objBB[$theKey]=$txtInfo;
-								$this->setup[$theKey.'.']['imgMap']=0;
+								$this->setup[$theKey.'.']['BBOX'] = $txtInfo;
+								$this->objBB[$theKey] = $txtInfo;
+								$this->setup[$theKey.'.']['imgMap'] = 0;
 							}
 						break;
 						case 'IMAGE':
 							$fileInfo = $this->getResource($conf['file'],$conf['file.']);
 							if ($fileInfo)	{
-								$this->setup[$theKey.'.']['file']=$fileInfo[3];
-								$this->setup[$theKey.'.']['BBOX']=$fileInfo;
-								$this->objBB[$theKey]=$fileInfo;
+								$this->combinedFileNames[] = ereg_replace('\.[[:alnum:]]+$','',basename($fileInfo[3]));
+								$this->setup[$theKey.'.']['file'] = $fileInfo[3];
+								$this->setup[$theKey.'.']['BBOX'] = $fileInfo;
+								$this->objBB[$theKey] = $fileInfo;
 								if ($conf['mask'])	{
 									$maskInfo = $this->getResource($conf['mask'],$conf['mask.']);
 									if ($maskInfo)	{
-										$this->setup[$theKey.'.']['mask']=$maskInfo[3];
+										$this->setup[$theKey.'.']['mask'] = $maskInfo[3];
 									} else {
 										$this->setup[$theKey.'.']['mask'] = '';
 									}
@@ -326,13 +348,14 @@ class tslib_gifBuilder extends t3lib_stdGraphic {
 		$XY = $this->XY;
 
 			// Gif-start
-		$this->im = imagecreate($XY[0],$XY[1]);
+		$this->im = $this->imagecreate($XY[0],$XY[1]);
 		$this->w = $XY[0];
 		$this->h = $XY[1];
 
 			// backColor is set
-		$cols=$this->convertColor($this->setup['backColor']);
-		ImageColorAllocate($this->im, $cols[0],$cols[1],$cols[2]);
+		$BGcols = $this->convertColor($this->setup['backColor']);
+		$Bcolor = ImageColorAllocate($this->im, $BGcols[0],$BGcols[1],$BGcols[2]);
+		ImageFilledRectangle($this->im, 0, 0, $XY[0], $XY[1], $Bcolor);
 
 			// Traverse the GIFBUILDER objects an render each one:
 		if (is_array($this->setup))	{
@@ -413,28 +436,20 @@ class tslib_gifBuilder extends t3lib_stdGraphic {
 				}
 			}
 		}
-			// Auto transparent background is set
+
+
 		if ($this->setup['transparentBackground'])	{
-			imagecolortransparent($this->im, imagecolorat($this->im, 0, 0));
-		}
-			// TransparentColors are set
-		if (is_array($this->setup['transparentColor_array']))	{
-			reset($this->setup['transparentColor_array']);
-			while(list(,$transparentColor)=each($this->setup['transparentColor_array']))	{
-				$cols=$this->convertColor($transparentColor);
-				if ($this->setup['transparentColor.']['closest'])	{
-					$colIndex = ImageColorClosest ($this->im, $cols[0],$cols[1],$cols[2]);
-				} else {
-					$colIndex = ImageColorExact ($this->im, $cols[0],$cols[1],$cols[2]);
-				}
-				if ($colIndex > -1) {
-					ImageColorTransparent($this->im, $colIndex);
-				} else {
-					ImageColorTransparent($this->im, ImageColorAllocate($this->im, $cols[0],$cols[1],$cols[2]));
-				}
-				break;		// Originally we thought of letting many colors be defined as transparent, but GDlib seems to accept only one definition. Therefore we break here. Maybe in the future this 'break' will be cancelled if a method of truly defining many transparent colors could be found.
+				// Auto transparent background is set
+			$Bcolor = ImageColorExact($this->im, $BGcols[0],$BGcols[1],$BGcols[2]);
+			imagecolortransparent($this->im, $Bcolor);
+		} elseif (is_array($this->setup['transparentColor_array']))	{
+				// Multiple transparent colors are set. This is done via the trick that all transparent colors get converted to one color and then this one gets set as transparent as png/gif can just have one transparent color.
+			$Tcolor = $this->unifyColors($this->im, $this->setup['transparentColor_array'], intval($this->setup['transparentColor.']['closest']));
+			if ($Tcolor>=0)	{
+				imagecolortransparent($this->im, $Tcolor);
 			}
 		}
+
 	}
 
 
@@ -474,7 +489,7 @@ class tslib_gifBuilder extends t3lib_stdGraphic {
 	 */
 	function checkTextObj($conf)	{
 		$conf['fontFile']=$this->checkFile($conf['fontFile']);
-		if (!$conf['fontFile']){$conf['fontFile']='t3lib/fonts/arial.ttf';}
+		if (!$conf['fontFile']){$conf['fontFile']='t3lib/fonts/nimbus.ttf';}
 		if (!$conf['iterations']){$conf['iterations'] = 1;}
 		if (!$conf['fontSize']){$conf['fontSize']=12;}
 		if ($conf['spacing'] || $conf['wordSpacing'])	{		// If any kind of spacing applys, we cannot use angles!!
@@ -484,11 +499,14 @@ class tslib_gifBuilder extends t3lib_stdGraphic {
 		$cObj =t3lib_div::makeInstance('tslib_cObj');
 		$cObj->start($this->data);
 
+		$conf['fontColor'] = trim($cObj->stdWrap($conf['fontColor'], $conf['fontColor.']));
 		$conf['text']=$cObj->stdWrap($conf['text'],$conf['text.']);
 			// Strip HTML
 		if (!$conf['doNotStripHTML'])	{
 			$conf['text'] = strip_tags($conf['text']);
 		}
+		$this->combinedTextStrings[] = strip_tags($conf['text']);
+
 			// Max length = 100
 		$tlen = intval($conf['textMaxLength']) ? intval($conf['textMaxLength']) : 100;
 		$conf['text'] = substr($conf['text'],0,$tlen);
@@ -599,7 +617,9 @@ class tslib_gifBuilder extends t3lib_stdGraphic {
 	 * @link http://typo3.org/doc.0.html?&tx_extrepmgm_pi1[extUid]=270&tx_extrepmgm_pi1[tocEl]=282&cHash=831a95115d
 	 */
 	function getResource($file,$fileArray)	{
-		$fileArray['ext']= $this->gifExtension;
+		if (!t3lib_div::inList($this->imageFileExt, $fileArray['ext']))	{
+			$fileArray['ext'] = $this->gifExtension;
+		}
 		$cObj =t3lib_div::makeInstance('tslib_cObj');
 		$cObj->start($this->data);
 		return $cObj->getImgResource($file,$fileArray);
@@ -625,7 +645,13 @@ class tslib_gifBuilder extends t3lib_stdGraphic {
 	 * @access private
 	 */
 	function fileName($pre)	{
-		return $this->tempPath.$pre.t3lib_div::shortMD5(serialize($this->setup)).'.'.$this->extension();
+
+			// WARNING: In PHP5 I discovered that rendering with freetype of Japanese letters was totally corrupt. Not only the wrong glyphs are printed but also some memory stack overflow resulted in strange additional chars - and finally the reason for this investigation: The Bounding box data was changing all the time resulting in new images being generated all the time. With PHP4 it works fine.
+		return $this->tempPath.
+				$pre.
+				($GLOBALS['TSFE']->config['config']['meaningfulTempFilePrefix'] ? $GLOBALS['TSFE']->fileNameASCIIPrefix(implode('_',array_merge($this->combinedTextStrings,$this->combinedFileNames)),intval($GLOBALS['TSFE']->config['config']['meaningfulTempFilePrefix']),'_') : '').
+				t3lib_div::shortMD5(serialize($this->setup)).
+				'.'.$this->extension();
 	}
 
 	/**
@@ -640,6 +666,12 @@ class tslib_gifBuilder extends t3lib_stdGraphic {
 			case 'jpeg':
 				return 'jpg';
 			break;
+			case 'png':
+				return 'png';
+			break;
+			case 'gif':
+				return 'gif';
+			break;
 			default:
 				return $this->gifExtension;
 			break;
