@@ -2,7 +2,7 @@
 /***************************************************************
 *  Copyright notice
 *
-*  (c) 1999-2004 Kasper Skaarhoj (kasperYYYY@typo3.com)
+*  (c) 1999-2008 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
@@ -27,7 +27,7 @@
 /**
  * Contains the TMENU_LAYERS extension class, tslib_tmenu_layers
  *
- * $Id: tmenu_layers.php,v 1.7 2004/09/13 22:57:37 typo3 Exp $
+ * $Id: tmenu_layers.php 3439 2008-03-16 19:16:51Z flyguide $
  * Revised for TYPO3 3.6 June/2003 by Kasper Skaarhoj
  * XHTML compliant
  *
@@ -39,14 +39,14 @@
  *
  *
  *   84: class tslib_tmenu_layers extends tslib_tmenu
- *  117:     function extProc_init()
- *  135:     function extProc_RO($key)
- *  151:     function extProc_beforeLinking($key)
- *  202:     function extProc_afterLinking($key)
- *  237:     function extProc_beforeAllWrap($item,$key)
- *  250:     function isSetIntval($in)
- *  259:     function extProc_finish ()
- *  435:     function extCalcBorderWithin($kind,$integer)
+ *  116:     function extProc_init()
+ *  134:     function extProc_RO($key)
+ *  150:     function extProc_beforeLinking($key)
+ *  205:     function extProc_afterLinking($key)
+ *  240:     function extProc_beforeAllWrap($item,$key)
+ *  253:     function isSetIntval($in)
+ *  262:     function extProc_finish ()
+ *  444:     function extCalcBorderWithin($kind,$integer)
  *
  * TOTAL FUNCTIONS: 8
  * (This index is automatically created/updated by the extension "extdeveval")
@@ -73,7 +73,7 @@
 
 /**
  * Class extending tslib_tmenu for the creation of text based DHTML menus
- * NOTICE: The contents of this class is copied EXACTLY AS IS from gmenu_layers class! See noties in class (for BEGIN/END) and also 'diff.xmenu_layers.txt'
+ * NOTICE: The contents of this class is copied EXACTLY AS IS from gmenu_layers class! See notes in class (for BEGIN/END) and also 'diff.xmenu_layers.txt'
  *
  * @author	Kasper Skaarhoj <kasperYYYY@typo3.com>
  * @package TYPO3
@@ -83,7 +83,6 @@
  */
 class tslib_tmenu_layers extends tslib_tmenu {
 
-
 // FULL DUPLICATE FROM gmenu_layers BEGIN:
 
 	var $GMENU_fixKey='layers';
@@ -152,7 +151,7 @@ class tslib_tmenu_layers extends tslib_tmenu {
 		if ($this->I['uid'])	{
 
 			array_push($GLOBALS['TSFE']->applicationData['GMENU_LAYERS']['WMparentId'],$this->WMid);
-			$this->WMtheSubMenu = $this->subMenu($this->I['uid']);
+			$this->WMtheSubMenu = $this->subMenu($this->I['uid'], $this->WMsubmenuObjSuffixes[$key]['sOSuffix']);
 			array_pop($GLOBALS['TSFE']->applicationData['GMENU_LAYERS']['WMparentId']);
 			$this->WMisSub = trim($this->WMtheSubMenu) ? 1 : 0;
 
@@ -177,6 +176,8 @@ GLV_restoreMenu["'.$this->WMid.'"] = "'.$this->WMactiveKey.'";
 			if ($this->WMisSub)	{
 				$event="GL_stopMove('".$this->WMid."');";
 				$this->I['linkHREF']['onMouseover']='GL_doTop(\''.$this->WMid.'\', \'Menu'.$this->WMid.$key.'\');'.$this->I['linkHREF']['onMouseover'];
+					// IESelectFix - Activates IFRAME layer below menu
+				if ($this->mconf['ieSelectFix']) $this->I['linkHREF']['onMouseover']=$this->I['linkHREF']['onMouseover'].'GL_iframer(\''.$this->WMid.'\',\'Menu'.$this->WMid.$key.'\',true);';
 					// Added 120802; This means that everytime leaving a menuitem the layer should be shut down (and if the layer is hit in the meantime it is not though).
 					// This should happen only for items that are auto-hidden when not over and possibly only when a hide-timer is set. Problem is if the hide-timer is not set and we leave the main element, then the layer will be hidden unless we reach the layer before the timeout will happen and the menu hidden.
 				if (t3lib_div::intInRange($this->mconf['hideMenuWhenNotOver'],0,600) && $this->mconf['hideMenuTimer'])	{
@@ -184,7 +185,9 @@ GLV_restoreMenu["'.$this->WMid.'"] = "'.$this->WMactiveKey.'";
 				}
 				$this->I['linkHREF']['onMouseout'].=$event;
 			} else {
-				$this->I['linkHREF']['onMouseover']='GL_hideAll("'.$this->WMid.'");'.$this->I['linkHREF']['onMouseover'];
+				$this->I['linkHREF']['onMouseover'] = 'GL_hideAll("'.$this->WMid.'");'.$this->I['linkHREF']['onMouseover'];
+					// IESelectFix - Hides IFRAME layer below menu
+				if ($this->mconf['ieSelectFix']) $this->I['linkHREF']['onMouseover'] = $this->I['linkHREF']['onMouseover'].'GL_iframer(\''.$this->WMid.'\',\'\',false);';
 				$event='GL_resetAll("'.$this->WMid.'");';
 				$this->I['linkHREF']['onMouseout'].=$event;
 			}
@@ -343,9 +346,15 @@ GL_restoreMenu("'.$mIdStr.'");';
 				$this->WMresetSubMenus.='
 if (!GLV_doReset["'.$mIdStr.'"] && GLV_currentLayer["'.$mIdStr.'"]!=null)	resetSubMenu=0;';
 			}
+		}
+			// IESelectFix - Adds IFRAME tag to HTML, Hides IFRAME layer below menu
+		if ($this->mconf['ieSelectFix']) {
+			$this->WMhideCode.= '
+	GL_iframer(\''.$this->WMid.'\',\'\',false);';
+			$this->divLayers['iframe'] = '<iframe id="Iframe'.$this->WMid.'" scrolling="no" frameborder="0" style="position:absolute; top:0px; left:0px; background-color:transparent; layer-background-color:transparent; display:none;"></iframe>';
 		}
 		$GLOBALS['TSFE']->applicationData['GMENU_LAYERS']['WMid']=array_merge($this->WMtempStore,$GLOBALS['TSFE']->applicationData['GMENU_LAYERS']['WMid']);
-		$GLOBALS['TSFE']->additionalHeaderData['gmenu_layer_shared']='<script type="text/javascript" src="'.$GLOBALS['TSFE']->absRefPrefix.'media/scripts/jsfunc.layermenu.js"></script>';
+		$GLOBALS['TSFE']->additionalHeaderData['gmenu_layer_shared']='<script type="text/javascript" src="'.t3lib_extMgm::siteRelPath('cms').'tslib/media/scripts/jsfunc.layermenu.js"></script>';
 		$GLOBALS['TSFE']->JSCode.= '
 
 GLV_curLayerWidth["'.$this->WMid.'"]=0;
