@@ -27,7 +27,7 @@
 /**
  * Contains a class with functions for page related overview of translations.
  *
- * $Id: class.tx_cms_webinfo_lang.php 3439 2008-03-16 19:16:51Z flyguide $
+ * $Id: class.tx_cms_webinfo_lang.php 4166 2008-09-23 08:44:14Z masi $
  *
  * @author	Kasper Skaarhoj <kasperYYYY@typo3.com>
  */
@@ -136,23 +136,21 @@ class tx_cms_webinfo_lang extends t3lib_extobjbase {
 			#debug($tree->tree);
 
 				// Add CSS needed:
-			$css_content = '
-				TABLE#langTable {
+			$this->pObj->doc->inDocStylesArray[] = '
+				table#langTable {
 					margin-top: 10px;
 				}
-				TABLE#langTable TR TD {
+				table#langTable tr td {
 					padding-left : 2px;
 					padding-right : 2px;
 					white-space: nowrap;
 				}
-				TD.c-blocked { background-color: red; }
-				TD.c-ok { background-color: #669966; }
-				TD.c-fallback {  }
-				TD.c-leftLine {border-left: 2px solid black; }
+				td.c-blocked { background-color: #f72116; }
+				td.c-ok { background-color: #86b571; }
+				td.c-fallback {  }
+				td.c-leftLine {border-left: 2px solid black; }
 				.bgColor5 { font-weight: bold; }
 			';
-			$marker = '/*###POSTCSSMARKER###*/';
-			$this->pObj->content = str_replace($marker,$css_content.chr(10).$marker,$this->pObj->content);
 
 				// Render information table:
 			$theOutput.= $this->renderL10nTable($tree);
@@ -338,9 +336,9 @@ class tx_cms_webinfo_lang extends t3lib_extobjbase {
 	 */
 	function getSystemLanguages()	{
 		if (!$GLOBALS['BE_USER']->user['admin'] &&
-			strlen($GLOBALS['BE_USER']->dataLists['allowed_languages'])) {
+			strlen($GLOBALS['BE_USER']->groupData['allowed_languages'])) {
 
-			$allowed_languages = array_flip(explode(',', $GLOBALS['BE_USER']->dataLists['allowed_languages']));
+			$allowed_languages = array_flip(explode(',', $GLOBALS['BE_USER']->groupData['allowed_languages']));
 		}
 
 		$res = $GLOBALS['TYPO3_DB']->exec_SELECTquery(
@@ -419,4 +417,5 @@ class tx_cms_webinfo_lang extends t3lib_extobjbase {
 if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/cms/web_info/class.tx_cms_webinfo_lang.php'])	{
 	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/cms/web_info/class.tx_cms_webinfo_lang.php']);
 }
-?>
+
+?>
\ No newline at end of file
