@@ -59,7 +59,10 @@ class tx_felogin_pi1 extends tslib_pibase {
 			// Loading TypoScript array into object variable:
 		$this->conf = $conf;
 		$this->uploadDir = 'uploads/tx_felogin/';
-
+			
+			// Loading default pivars
+		$this->pi_setPiVarDefaults();
+		
 			// Loading language-labels
 		$this->pi_loadLL();
 
@@ -80,6 +83,11 @@ class tx_felogin_pi1 extends tslib_pibase {
 		$this->logintype = t3lib_div::_GP('logintype');
 		$this->redirectUrl = t3lib_div::_GP('redirect_url');
 
+			// if config.typolinkLinkAccessRestrictedPages is set, the var is return_url
+		$returnUrl =  t3lib_div::_GP('return_url');  
+		if ($returnUrl) {
+			$this->redirectUrl = $returnUrl;	 
+		}
 
 
 			// Get Template
@@ -129,6 +137,8 @@ class tx_felogin_pi1 extends tslib_pibase {
 	  */
 	 protected function showForgot() {
 		$subpart = $this->cObj->getSubpart($this->template, '###TEMPLATE_FORGOT###');
+		$subpartArray = $linkpartArray = array();
+
 		if ($this->piVars['forgot_email']) {
 			if (t3lib_div::validEmail($this->piVars['forgot_email'])) {
 					// look for user record and send the password
@@ -158,7 +168,7 @@ class tx_felogin_pi1 extends tslib_pibase {
 				}
 
 				$this->cObj->sendNotifyEmail($msg, $this->piVars['forgot_email'], '', $this->conf['email_from'], $this->conf['email_fromName'], $this->conf['replyTo']);
-				$markerArray['###STATUS_MESSAGE###'] = sprintf($this->pi_getLL('ll_forgot_message_emailSent', '', 1),'<em>' . htmlspecialchars($this->piVars['forgot_email']) .'</em>');
+				$markerArray['###STATUS_MESSAGE###'] = $this->cObj->stdWrap(sprintf($this->pi_getLL('ll_forgot_message_emailSent', '', 1), '<em>' . htmlspecialchars($this->piVars['forgot_email']) .'</em>'), $this->conf['forgotMessage_stdWrap.']);
 				$subpartArray['###FORGOT_FORM###'] = '';
 
 
@@ -192,6 +202,7 @@ class tx_felogin_pi1 extends tslib_pibase {
 	 */
 	protected function showLogout() {
 		$subpart = $this->cObj->getSubpart($this->template, '###TEMPLATE_LOGOUT###');
+		$subpartArray = $linkpartArray = array();
 
 		$markerArray['###STATUS_HEADER###'] = $this->getDisplayText('status_header',$this->conf['logoutHeader_stdWrap.']);
 		$markerArray['###STATUS_MESSAGE###']=$this->getDisplayText('status_message',$this->conf['logoutMessage_stdWrap.']);$this->cObj->stdWrap($this->flexFormValue('message','s_status'),$this->conf['logoutMessage_stdWrap.']);
@@ -204,6 +215,11 @@ class tx_felogin_pi1 extends tslib_pibase {
 		$markerArray['###USERNAME###'] = htmlspecialchars($GLOBALS['TSFE']->fe_user->user['username']);
 		$markerArray['###USERNAME_LABEL###'] = $this->pi_getLL('username', '', 1);
 
+		if ($this->redirectUrl) {
+				// use redirectUrl for action tag because of possible access restricted pages
+			$markerArray['###ACTION_URI###'] = htmlspecialchars($this->redirectUrl);
+			$this->redirectUrl = '';
+		}
 		return $this->cObj->substituteMarkerArrayCached($subpart, $markerArray, $subpartArray, $linkpartArray);
 	}
 
@@ -214,6 +230,9 @@ class tx_felogin_pi1 extends tslib_pibase {
 	 */
 	 protected function showLogin() {
 		$subpart = $this->cObj->getSubpart($this->template, '###TEMPLATE_LOGIN###');
+		$subpartArray = $linkpartArray = array();
+
+		$gpRedirectUrl = ''; 
 
 		$markerArray['###LEGEND###'] = $this->pi_getLL('oLabel_header_welcome', '', 1);
 
@@ -238,6 +257,7 @@ class tx_felogin_pi1 extends tslib_pibase {
 					// login error
 				$markerArray['###STATUS_HEADER###'] = $this->getDisplayText('error_header',$this->conf['errorHeader_stdWrap.']);
 				$markerArray['###STATUS_MESSAGE###'] = $this->getDisplayText('error_message',$this->conf['errorMessage_stdWrap.']);
+				$gpRedirectUrl = t3lib_div::_GP('redirect_url');
 			}
 		} else {
 			if($this->logintype === 'logout') {
@@ -275,13 +295,13 @@ class tx_felogin_pi1 extends tslib_pibase {
 			// Login form
 		$markerArray['###ACTION_URI###'] = $this->getPageLink('',array(),true);
 		$markerArray['###EXTRA_HIDDEN###'] = $extraHidden; // used by kb_md5fepw extension...
-        $markerArray['###LEGEND###'] = $this->pi_getLL('login', '', 1);
+		$markerArray['###LEGEND###'] = $this->pi_getLL('login', '', 1);
 		$markerArray['###LOGIN_LABEL###'] = $this->pi_getLL('login', '', 1);
 		$markerArray['###ON_SUBMIT###'] = $onSubmit; // used by kb_md5fepw extension...
 		$markerArray['###PASSWORD_LABEL###'] = $this->pi_getLL('password', '', 1);
 		$markerArray['###STORAGE_PID###'] = $this->spid;
 		$markerArray['###USERNAME_LABEL###'] = $this->pi_getLL('username', '', 1);
-		$markerArray['###REDIRECT_URL###'] = $this->redirectUrl;
+		$markerArray['###REDIRECT_URL###'] = $gpRedirectUrl ? htmlspecialchars($gpRedirectUrl) : htmlspecialchars($this->redirectUrl);
 
 		if ($this->flexFormValue('showForgotPassword','sDEF') || $this->conf['showForgotPasswordLink']) {
 			$linkpartArray['###FORGOT_PASSWORD_LINK###'] = explode('|',$this->getPageLink('|',array($this->prefixId.'[forgot]'=>1)));
@@ -391,7 +411,7 @@ class tx_felogin_pi1 extends tslib_pibase {
 				} elseif ($this->logintype === 'logout') { // after logout
 
 					// Hook for general actions after after logout has been confirmed
-					if ($GLOBALS['TYPO3_CONF_VARS']['EXTCONF']['felogin']['logout_confirmed']) {
+					if ($this->logintype === 'logout' && $GLOBALS['TYPO3_CONF_VARS']['EXTCONF']['felogin']['logout_confirmed']) {
 						$_params = array();
 						foreach($GLOBALS['TYPO3_CONF_VARS']['EXTCONF']['felogin']['logout_confirmed'] as $_funcRef) {
 							if ($_funcRef) {
@@ -410,6 +430,10 @@ class tx_felogin_pi1 extends tslib_pibase {
 				} else { // not logged in
 						// Placeholder for maybe future options
 					switch ($redirMethod) {
+						case 'getpost':
+							// preserve the get/post value
+							$redirect_url = $this->redirectUrl;
+						break;
 					}
 				}
 
