@@ -2,23 +2,32 @@
 /***************************************************************
 *  Copyright notice
 *
-*  (c) 1999-2004 Kasper Skaarhoj (kasperYYYY@typo3.com)
+*  (c) 1999-2008 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *
-*  This script is part of the TYPO3 project. TYPO3 is free software;
-*  You can redistribute it and/or modify it under the terms of the
-*  TYPO3 License as published from the www.typo3.com website.
+*  This script is part of the TYPO3 project. The TYPO3 project is
+*  free software; you can redistribute it and/or modify
+*  it under the terms of the GNU General Public License as published by
+*  the Free Software Foundation; either version 2 of the License, or
+*  (at your option) any later version.
+*
+*  The GNU General Public License can be found at
+*  http://www.gnu.org/copyleft/gpl.html.
+*  A copy is found in the textfile GPL.txt and important notices to the license
+*  from the author is found in LICENSE.txt distributed with these scripts.
+*
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
-*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+*  GNU General Public License for more details.
 *
-*  This copyright notice MUST APPEAR in all copies of this script
+*  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/
 /**
  * Contains the TYPO3 Backend Language class
  *
- * $Id: lang.php,v 1.15 2004/09/13 22:57:38 typo3 Exp $
+ * $Id: lang.php 3065 2008-02-04 22:23:37Z masi $
  * Revised for TYPO3 3.6.0
  *
  * @author	Kasper Skaarhoj <kasperYYYY@typo3.com>
@@ -28,22 +37,21 @@
  *
  *
  *
- *   80: class language
- *  128:     function init($lang,$altPath='')
- *  173:     function addModuleLabels($arr,$prefix)
- *  199:     function hscAndCharConv($lStr,$hsc)
- *  214:     function makeEntities($str)
- *  231:     function JScharCode($str)
- *  250:     function getLL($index,$hsc=0)
- *  267:     function getLLL($index,$LOCAL_LANG,$hsc=0)
- *  287:     function sL($input,$hsc=0)
- *  331:     function loadSingleTableDescription($table)
- *  383:     function includeLLFile($fileRef,$setGlobal=1,$mergeLocalOntoDefault=0)
- *  428:     function readLLfile($fileRef)
- *  449:     function readLLXMLfile($fileRef,$langKey)
- *  548:     function localizedFileRef($fileRef)
+ *   88: class language
+ *  138:     function init($lang,$altPath='')
+ *  183:     function addModuleLabels($arr,$prefix)
+ *  209:     function hscAndCharConv($lStr,$hsc)
+ *  224:     function makeEntities($str)
+ *  241:     function JScharCode($str)
+ *  260:     function getLL($index,$hsc=0)
+ *  278:     function getLLL($index,$LOCAL_LANG,$hsc=0)
+ *  299:     function sL($input,$hsc=0)
+ *  344:     function loadSingleTableDescription($table)
+ *  396:     function includeLLFile($fileRef,$setGlobal=1,$mergeLocalOntoDefault=0)
+ *  441:     function readLLfile($fileRef)
+ *  451:     function localizedFileRef($fileRef)
  *
- * TOTAL FUNCTIONS: 13
+ * TOTAL FUNCTIONS: 12
  * (This index is automatically created/updated by the extension "extdeveval")
  *
  */
@@ -94,6 +102,8 @@ class language {
 		'dk' => 'http://www.typo3.com/man_dk/',
 	);
 
+	var $debugKey = FALSE;		// If true, will show the key/location of labels in the backend.
+
 
 	var $moduleLabels = Array();	// Can contain labels and image references from the backend modules. Relies on t3lib_loadmodules to initialize modules after a global instance of $LANG has been created.
 
@@ -102,9 +112,12 @@ class language {
 	var $LL_files_cache=array();	// Internal cache for read LL-files
 	var $LL_labels_cache=array();	// Internal cache for ll-labels (filled as labels are requested)
 
-		// Internal charset conversion:
-	var $origCharSet='';		// If set, then it means that the this->charSet is set to a forced, common value for the WHOLE backend regardless of user language. And THIS variable will contain the original charset for the language labels. With ->csConvObj we must then convert the original charset to the charset used in the backend from now on.
-	var $csConvObj;				// An instance of the "t3lib_cs" class. May be used by any application.
+	/**
+	 * instance of the "t3lib_cs" class. May be used by any application.
+	 *
+	 * @var t3lib_cs
+	 */
+	var $csConvObj;
 
 
 
@@ -148,17 +161,12 @@ class language {
 			// If a forced charset is used and different from the charset otherwise used:
 		if ($GLOBALS['TYPO3_CONF_VARS']['BE']['forceCharset'] && $GLOBALS['TYPO3_CONF_VARS']['BE']['forceCharset']!=$this->charSet)	{
 				// Set the forced charset:
-			$this->origCharSet = $this->charSet;
 			$this->charSet = $GLOBALS['TYPO3_CONF_VARS']['BE']['forceCharset'];
 
-			if ($this->charSet!='utf-8' && !$this->csConvObj->initCharset($this->charSet))		{
+			if ($this->charSet!='utf-8' && !$this->csConvObj->initCharset($this->charSet))	{
 				t3lib_BEfunc::typo3PrintError ('The forced character set "'.$this->charSet.'" was not found in t3lib/csconvtbl/','Forced charset not found');
 				exit;
 			}
-			if ($this->origCharSet!='utf-8' && !$this->csConvObj->initCharset($this->origCharSet))		{
-				t3lib_BEfunc::typo3PrintError ('The original character set "'.$this->origCharSet.'" was not found in t3lib/csconvtbl/','Forced charset not found');
-				exit;
-			}
 		}
 	}
 
@@ -198,9 +206,9 @@ class language {
 	 */
 	function hscAndCharConv($lStr,$hsc)	{
 		$lStr = $hsc ? htmlspecialchars($lStr) : $lStr;
-		if ($this->origCharSet)	{
-			$lStr = $this->csConvObj->conv($lStr,$this->origCharSet,$this->charSet,1);
-		}
+
+		// labels returned from a locallang file used to be in the language of the charset. Since TYPO3 4.1 they are always in the charset of the BE.
+
 		return $lStr;
 	}
 
@@ -250,10 +258,11 @@ class language {
 	function getLL($index,$hsc=0)	{
 		// Get Local Language
 		if (strcmp($GLOBALS['LOCAL_LANG'][$this->lang][$index],''))	{
-			return $this->hscAndCharConv($GLOBALS['LOCAL_LANG'][$this->lang][$index], $hsc);	// Returns local label if not blank.
+			$output = $this->hscAndCharConv($GLOBALS['LOCAL_LANG'][$this->lang][$index], $hsc);	// Returns local label if not blank.
 		} else {
-			return $this->hscAndCharConv($GLOBALS['LOCAL_LANG']['default'][$index], $hsc);	// Returns default label
+			$output = $this->hscAndCharConv($GLOBALS['LOCAL_LANG']['default'][$index], $hsc);	// Returns default label
 		}
+		return $output.($this->debugKey ? ' ['.$index.']':'');
 	}
 
 	/**
@@ -267,17 +276,18 @@ class language {
 	function getLLL($index,$LOCAL_LANG,$hsc=0)	{
 		// Get Local Language
 		if (strcmp($LOCAL_LANG[$this->lang][$index],''))	{
-			return $this->hscAndCharConv($LOCAL_LANG[$this->lang][$index], $hsc);	// Returns local label if not blank.
+			$output = $this->hscAndCharConv($LOCAL_LANG[$this->lang][$index], $hsc);	// Returns local label if not blank.
 		} else {
-			return $this->hscAndCharConv($LOCAL_LANG['default'][$index], $hsc);		// Returns default label
+			$output = $this->hscAndCharConv($LOCAL_LANG['default'][$index], $hsc);		// Returns default label
 		}
+		return $output.($this->debugKey ? ' ['.$index.']':'');
 	}
 
 	/**
 	 * splitLabel function
 	 * Historically labels were exploded by '|' and each part would correspond to the translation of the language found at the same 'index' in the TYPO3_languages constant.
 	 * Today all translations are based on $LOCAL_LANG variables. 'language-splitted' labels can therefore refer to a local-lang file + index instead!
-	 * It's highly recommended to use the 'local_lang' method (and thereby it's highly depreciated to use 'language-splitted' label strings)
+	 * It's highly recommended to use the 'local_lang' method (and thereby it's highly deprecated to use 'language-splitted' label strings)
 	 * Refer to 'Inside TYPO3' for more details
 	 *
 	 * @param	string		Label key/reference
@@ -317,7 +327,8 @@ class language {
 				}
 				$this->LL_labels_cache[$this->lang][$input] = $this->getLLL($parts[1],$this->LL_files_cache[$parts[0]]);
 			}
-			return $hsc ? t3lib_div::deHSCentities(htmlspecialchars($this->LL_labels_cache[$this->lang][$input])) : $this->LL_labels_cache[$this->lang][$input]; // For the cached output charset conversion has already happend! So perform HSC right here.
+			$output = $hsc ? t3lib_div::deHSCentities(htmlspecialchars($this->LL_labels_cache[$this->lang][$input])) : $this->LL_labels_cache[$this->lang][$input]; // For the cached output charset conversion has already happend! So perform HSC right here.
+			return $output.($this->debugKey ? ' ['.$input.']':'');
 		}
 	}
 
@@ -391,7 +402,7 @@ class language {
 
 		if (count($llang))	{
 
-			$LOCAL_LANG = t3lib_div::array_merge_recursive_overrule($LOCAL_LANG,$llang);
+			$LOCAL_LANG = t3lib_div::array_merge_recursive_overrule((array)$LOCAL_LANG,$llang);
 
 				// Localized addition?
 			$lFileRef = $this->localizedFileRef($fileRef);
@@ -426,117 +437,7 @@ class language {
 	 * @return	array		Value of $LOCAL_LANG found in the included file. If that array is found it's returned. Otherwise an empty array
 	 */
 	function readLLfile($fileRef)	{
-		$file = t3lib_div::getFileAbsFileName($fileRef);
-		if ($file)	{
-			$baseFile = ereg_replace('\.(php|xml)$', '', $file);
-
-			if (@is_file($baseFile.'.xml'))	{
-				$LOCAL_LANG = $this->readLLXMLfile($baseFile.'.xml', $this->lang);
-			} elseif (@is_file($baseFile.'.php'))	{
-				include($baseFile.'.php');
-			} else die('Filereference, "'.$file.'", not found!');
-		}
-		return is_array($LOCAL_LANG)?$LOCAL_LANG:array();
-	}
-
-	/**
-	 * Includes a locallang-xml file and returns the $LOCAL_LANG array
-	 *
-	 * @param	string		Absolute reference to locallang-XML file
-	 * @param	string		TYPO3 language key, eg. "dk" or "de" or "default"
-	 * @return	array		LOCAL_LANG array in return.
-	 */
-	function readLLXMLfile($fileRef,$langKey)	{
-
-		if (@is_file($fileRef) && $langKey)	{
-
-				// Set charset:
-			$origCharset = $this->csConvObj->parse_charset($this->csConvObj->charSetArray[$langKey] ? $this->csConvObj->charSetArray[$langKey] : 'iso-8859-1');
-
-				// Cache file name:
-			$hashSource = substr($fileRef,strlen(PATH_site)).'|'.date('d-m-Y H:i:s',filemtime($fileRef));
-			$cacheFileName = PATH_site.'typo3temp/llxml/'.
-							#str_replace('_','',ereg_replace('^.*\/','',dirname($fileRef))).
-							#'_'.basename($fileRef).
-							substr(basename($fileRef),10,15).
-							'_'.t3lib_div::shortMD5($hashSource).'.'.$langKey.'.'.$origCharset.'.cache';
-
-				// Check if cache file exists...
-			if (!@is_file($cacheFileName))	{	// ... if it doesn't, create content and write it:
-
-					// Read XML, parse it.
-				$xmlString = t3lib_div::getUrl($fileRef);
-				$xmlContent = t3lib_div::xml2array($xmlString);
-
-					// Set default LOCAL_LANG array content:
-				$LOCAL_LANG = array();
-				$LOCAL_LANG['default'] = $xmlContent['data']['default'];
-
-					// Specific language, convert from utf-8 to backend language charset:
-					// NOTICE: Converting from utf-8 back to "native" language may be a temporary solution until we can totally discard "locallang.php" files altogether (and use utf-8 for everything). But doing this conversion is the quickest way to migrate now and the source is in utf-8 anyway which is the main point.
-				if ($langKey!='default')	{
-					$LOCAL_LANG[$langKey] = $xmlContent['data'][$langKey];
-
-						// Checking if charset should be converted.
-					if (is_array($LOCAL_LANG[$langKey]) && $origCharset!='utf-8')	{
-						foreach($LOCAL_LANG[$langKey] as $labelKey => $labelValue)	{
-							$LOCAL_LANG[$langKey][$labelKey] = $this->csConvObj->utf8_decode($labelValue,$origCharset);
-						}
-					}
-				}
-
-					// Cache the content now:
-				$serContent = array('origFile'=>$hashSource, 'LOCAL_LANG'=>$LOCAL_LANG);
-				$res = t3lib_div::writeFileToTypo3tempDir($cacheFileName, serialize($serContent));
-				if ($res)	die('ERROR: '.$res);
-			} else {
-					// Get content from cache:
-				$serContent = unserialize(t3lib_div::getUrl($cacheFileName));
-				$LOCAL_LANG = $serContent['LOCAL_LANG'];
-			}
-
-				// Checking for EXTERNAL file for non-default language:
-			if ($langKey!='default' && is_string($LOCAL_LANG[$langKey]) && strlen($LOCAL_LANG[$langKey]))	{
-
-					// Look for localized file:
-				$localized_file = t3lib_div::getFileAbsFileName($LOCAL_LANG[$langKey]);
-				if ($localized_file && @is_file($localized_file))	{
-
-						// Cache file name:
-					$hashSource = substr($localized_file,strlen(PATH_site)).'|'.date('d-m-Y H:i:s',filemtime($localized_file));
-					$cacheFileName = PATH_site.'typo3temp/llxml/ext_'.
-									substr(basename($localized_file),10,15).
-									'_'.t3lib_div::shortMD5($hashSource).'.'.$langKey.'.'.$origCharset.'.cache';
-
-						// Check if cache file exists...
-					if (!@is_file($cacheFileName))	{	// ... if it doesn't, create content and write it:
-
-							// Read and parse XML content:
-						$local_xmlString = t3lib_div::getUrl($localized_file);
-						$local_xmlContent = t3lib_div::xml2array($local_xmlString);
-						$LOCAL_LANG[$langKey] = is_array($local_xmlContent['data'][$langKey]) ? $local_xmlContent['data'][$langKey] : array();
-
-							// Checking if charset should be converted.
-						if (is_array($LOCAL_LANG[$langKey]) && $origCharset!='utf-8')	{
-							foreach($LOCAL_LANG[$langKey] as $labelKey => $labelValue)	{
-								$LOCAL_LANG[$langKey][$labelKey] = $this->csConvObj->utf8_decode($labelValue,$origCharset);
-							}
-						}
-
-							// Cache the content now:
-						$serContent = array('extlang'=>$langKey, 'origFile'=>$LOCAL_LANG[$langKey], 'EXT_DATA'=>$LOCAL_LANG[$langKey]);
-						$res = t3lib_div::writeFileToTypo3tempDir($cacheFileName, serialize($serContent));
-						if ($res)	die('ERROR: '.$res);
-					} else {
-							// Get content from cache:
-						$serContent = unserialize(t3lib_div::getUrl($cacheFileName));
-						$LOCAL_LANG[$langKey] = $serContent['EXT_DATA'];
-					}
-				}
-			}
-
-			return $LOCAL_LANG;
-		}
+		return t3lib_div::readLLfile($fileRef, $this->lang, $this->charSet);
 	}
 
 	/**
