@@ -2,7 +2,7 @@
 /***************************************************************
 *  Copyright notice
 *
-*  (c) 1999-2004 Kasper Skaarhoj (kasperYYYY@typo3.com)
+*  (c) 1999-2008 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
@@ -27,7 +27,7 @@
 /**
  * Wizard to help make forms (fx. for tt_content elements) of type 'form'.
  *
- * $Id: wizard_forms.php,v 1.18 2004/09/13 22:57:22 typo3 Exp $
+ * $Id: wizard_forms.php 3439 2008-03-16 19:16:51Z flyguide $
  * Revised for TYPO3 3.6 November/2003 by Kasper Skaarhoj
  * XHTML compliant
  *
@@ -41,17 +41,17 @@
  *  175: class SC_wizard_forms
  *  203:     function init()
  *  242:     function main()
- *  258:     function printContent()
- *  267:     function formsWizard()
+ *  257:     function printContent()
+ *  268:     function formsWizard()
  *
  *              SECTION: Helper functions
- *  310:     function getConfigCode(&$row)
- *  381:     function getFormHTML($formCfgArray,$row)
- *  660:     function changeFunc()
- *  719:     function cfgArray2CfgString($cfgArr)
- *  801:     function cfgString2CfgArray($cfgStr)
- *  900:     function cleanT($tArr)
- *  918:     function formatCells($fArr)
+ *  311:     function getConfigCode(&$row)
+ *  382:     function getFormHTML($formCfgArray,$row)
+ *  662:     function changeFunc()
+ *  721:     function cfgArray2CfgString($cfgArr)
+ *  803:     function cfgString2CfgArray($cfgStr)
+ *  902:     function cleanT($tArr)
+ *  920:     function formatCells($fArr)
  *
  * TOTAL FUNCTIONS: 11
  * (This index is automatically created/updated by the extension "extdeveval")
@@ -175,7 +175,12 @@ $LANG->includeLLFile('EXT:lang/locallang_wizards.xml');
 class SC_wizard_forms {
 
 		// Internal, dynamic:
-	var $doc;					// Document template object
+	/**
+	 * document template object
+	 *
+	 * @var mediumDoc
+	 */
+	var $doc;
 	var $content;				// Content accumulation for the module.
 	var $include_once=array();	// List of files to include.
 	var $attachmentCounter = 0;	// Used to numerate attachments automatically.
@@ -217,7 +222,7 @@ class SC_wizard_forms {
 		$this->doc->backPath = $BACK_PATH;
 		$this->doc->JScode=$this->doc->wrapScriptTags('
 			function jumpToUrl(URL,formEl)	{	//
-				document.location = URL;
+				window.location.href = URL;
 			}
 		');
 
@@ -247,7 +252,6 @@ class SC_wizard_forms {
 		} else {
 			$this->content.=$this->doc->section($LANG->getLL('forms_title'),'<span class="typo3-red">'.$LANG->getLL('table_noData',1).'</span>',0,1);
 		}
-		$this->content.=$this->doc->endPage();
 	}
 
 	/**
@@ -256,6 +260,8 @@ class SC_wizard_forms {
 	 * @return	void
 	 */
 	function printContent()	{
+		$this->content.= $this->doc->endPage();
+		$this->content = $this->doc->insertStylesAndJS($this->content);
 		echo $this->content;
 	}
 
@@ -267,7 +273,7 @@ class SC_wizard_forms {
 	function formsWizard()	{
 
 			// First, check the references by selecting the record:
-		$row=t3lib_BEfunc::getRecord($this->P['table'],$this->P['uid']);
+ 		$row = t3lib_BEfunc::getRecord($this->P['table'],$this->P['uid']);
 		if (!is_array($row))	{
 			t3lib_BEfunc::typo3PrintError ('Wizard Error','No reference to record',0);
 			exit;
@@ -318,7 +324,7 @@ class SC_wizard_forms {
 				// Convert to string (either line based or XML):
 			if ($this->xmlStorage)	{
 					// Convert the input array to XML:
-				$bodyText = t3lib_div::array2xml($this->FORMCFG['c'],'',0,'T3FormWizard');
+				$bodyText = t3lib_div::array2xml_cs($this->FORMCFG['c'],'T3FormWizard');
 
 					// Setting cfgArr directly from the input:
 				$cfgArr = $this->FORMCFG['c'];
@@ -507,7 +513,7 @@ class SC_wizard_forms {
 					$ctrl='';
 					$onClick="document.wizardForm.action+='#ANC_".(($k+1)*2-2)."';";
 					$onClick=' onclick="'.htmlspecialchars($onClick).'"';
-
+// FIXME $inputStyle undefined
 					$brTag=$inputStyle?'':'<br />';
 					if ($k!=0)	{
 						$ctrl.='<input type="image" name="FORMCFG[row_up]['.(($k+1)*2).']"'.t3lib_iconWorks::skinImg($this->doc->backPath,'gfx/pil2up.gif','').$onClick.' title="'.$LANG->getLL('table_up',1).'" />'.$brTag;
@@ -515,6 +521,7 @@ class SC_wizard_forms {
 						$ctrl.='<input type="image" name="FORMCFG[row_bottom]['.(($k+1)*2).']"'.t3lib_iconWorks::skinImg($this->doc->backPath,'gfx/turn_up.gif','').$onClick.' title="'.$LANG->getLL('table_bottom',1).'" />'.$brTag;
 					}
 					$ctrl.='<input type="image" name="FORMCFG[row_remove]['.(($k+1)*2).']"'.t3lib_iconWorks::skinImg($this->doc->backPath,'gfx/garbage.gif','').$onClick.' title="'.$LANG->getLL('table_removeRow',1).'" />'.$brTag;
+// FIXME $tLines undefined
 					if (($k+1)!=count($tLines))	{
 						$ctrl.='<input type="image" name="FORMCFG[row_down]['.(($k+1)*2).']"'.t3lib_iconWorks::skinImg($this->doc->backPath,'gfx/pil2down.gif','').$onClick.' title="'.$LANG->getLL('table_down',1).'" />'.$brTag;
 					} else {
